﻿/// <reference path="jquery-vsdoc.js" />
/// <reference path="dq.js" />
/// <reference path="../ui/jquery-ui.js" />
/// <reference path="../ui/ui.core.js" />
/// <reference path="../ui/ui.draggable.js" />
/// <reference path="../ui/ui.resizable.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0002
*/

DQ.Timer = function () {
    var me = this;
    var timerID = -1;
    DQ.Timer.start = function () {
        timerID = setInterval(me._interval, function () { me.onTick && me.onTick(); });
    }
    DQ.Timer.stop = function () {
        clearInterval(timerID);
        timerID = -1;
    }
}
DQ.Timer.prototype.enabled = __dqproperty__(DQ.Timer, "enabled",
    function () {
        return this._enabled;
    },
    function (value) {
        this._enabled = value;
    },
    false
);
DQ.Timer.prototype.interval = __dqproperty__(DQ.Timer, "interval",
    function () {
        return this._interval;
    },
    function (value) {
        this._interval = value;
    },
    1000
);

DQ.Selector = function (parent, options) {
    /// <summary>範囲選択枠を操作します</summary>
    options = DQ.options(options, {
        x: 0,
        y: 0,
        dx: 0,
        dy: 0,
        enabled: false,
        grid: [DQ.RESOLUTION, DQ.RESOLUTION],
        onselected: null
    });
    this._create(parent, options);
    return this;
}

DQ.Selector.prototype = {
    x: 0,
    y: 0,
    dx: 0,
    dy: 0,
    grid: [DQ.RESOLUTION, DQ.RESOLUTION],
    onselected: null,
    dragging: false,
    _enabled: true,
    _create: function (parent, options) {
        options = options || {};
        this.screen = parent;
        var pt = this.screen.obj;
        this.obj = $('<div>').addClass('dq-selector');
        this.obj.appendTo(pt).hide();
        this.grid = options.grid || [DQ.RESOLUTION, DQ.RESOLUTION];
        this.onselected = options.onselected || null;
        this.enabled(options.enabled);
    },
    _dragStart: function (e) {
        if (!e.data.enabled) {
            return true;
        }
        var me = this;
        var toClientX = function (pos) {
            var p = me.screen.obj[0].offsetParent;
            var v = me.screen.obj[0].offsetLeft;
            while (p) {
                v += p.offsetLeft;
                p = p.offsetParent;
            }
            return pos + me.screen.obj[0].scrollLeft - v;
        },
        toClientY = function (pos) {
            var p = me.screen.obj[0].offsetParent;
            var v = me.screen.obj[0].offsetTop;
            while (p) {
                v += p.offsetTop;
                p = p.offsetParent;
            }
            return pos + me.screen.obj[0].scrollTop - v;
        }

        var me = e.data;
        var g = me.obj.show();
        var cx = toClientX(e.pageX);
        var cy = toClientY(e.pageY);
        gx = parseInt(g.css('left'));
        gy = parseInt(g.css('top'));
        gw = g[0].clientWidth + 3;
        gh = g[0].clientHeight + 3;

        var dx = this.dx = 1;
        var dy = this.dy = 1;
        var r = me.pt;
        var rx = Math.floor(cx / me.grid[0]) * me.grid[0];
        var ry = Math.floor(cy / me.grid[1]) * me.grid[1];

        me.x = e.screenX - (cx - rx);
        me.y = e.screenY - (cy - ry);

        //あたり判定(セレクター内なら無視
        if (me.dragging && Math.abs(rx - gx) < gw && Math.abs(ry - gy) < gh) {
            return;
        }

        me.dragging = true;
        var wo = parseInt(g.css('border-width')) * 2 || 4;
        g.css({ 'top': ry + 'px', 'left': rx + 'px' });
        g.css({ 'width': dx * me.grid[0] - wo + 'px', 'height': dy * me.grid[1] - wo + 'px' }).show();

        g.resizable && g.resizable('destroy');

        $(document).bind("mousemove", me, me._move);
        $(document).one("mouseup", me, me._dragEnd);

        return false;
    },
    _dragEnd: function (e) {
        var me = e.data;
        var g = me.obj;

        me.dragging = false;
        $(document).unbind("mousemove", me._move);
        g.resizable && g.resizable({ grid: me.grid });

        me.onselected && me.onselected.apply(me);

        return false;
    },
    _move: function (e) {
        var me = e.data;
        var ldx = Math.floor((e.screenX - me.x) / me.grid[0]) + 1;
        var ldy = Math.floor((e.screenY - me.y) / me.grid[1]) + 1;
        ldx = ldx < 1 ? 1 : ldx;
        ldy = ldy < 1 ? 1 : ldy;
        if (ldx != (me.dx + 1) || ldy != (me.dy + 1)) {
            var wo = parseInt(me.obj.css('border-width')) * 2 || 4;
            me.obj.css({ 'width': ldx * me.grid[0] - wo + 'px', 'height': ldy * me.grid[1] - wo + 'px' });
            me.dx = ldx;
            me.dy = ldy;
        }

        return false;
    },
    enabled: function (value) {
        if (!arguments.length) {
            return this._enabled;
        }
        if (value == this._enabled) {
            return;
        }
        var me = this;
        if (value) {
            this.screen.obj.bind("mousedown", me, me._dragStart);
        } else {
            this.screen.obj.unbind("mousedown", me._dragStart);
        }
        this._enabled = value;
        return this._enabled;
    },
    hide: function () {
        this.obj.hide();
        this.obj.css({ 'top': '0px', 'left': '0px' });
        this.obj.css({ 'width': '1px', 'height': '1px' });
    },
    dispose: function () {
        this.obj.unbind("mousedown", this._dragStart);
        this.screen.remove(this);
    },
    pickUp: function () {
        /// <summary>
        /// 選択範囲内のグリッド単位の座標一覧を返します
        /// </summary>
        var res = new Array();
        var me = this;
        var g = me.obj;
        gx = parseInt(g.css('left')) / this.grid[0] + 1;
        gy = parseInt(g.css('top')) / this.grid[1] + 1;
        gw = g[0].clientWidth / me.grid[0];
        gh = g[0].clientHeight / me.grid[1];
        for (var y = gy; y < gy + gh; y++) {
            for (var x = gx; x < gx + gw; x++) {
                res.push([x, y]);
            }
        }

        return res;
    }
}

DQ.saveCatalog = function (catalog, project, path, callback) {
    var sel = new DQ.JsonSerializer(),
                json = sel.serialize(catalog);

    if (false) {
        DQ.UI.MessageBox.show(json, "更新結果", DQ.UI.MessageBox.Buttons.OK, DQ.UI.MessageBox.Icon.Asterisk);
    } else {
        DQ.__xhr__ = $.ajax({
            type: "POST",
            url: "mapEditor.php5",
            data: { project: project, data: json, path: path, command: 'save' },
            success: function (msg) {
                if (callback) {
                    callback && callback();
                } else {
                    DQ.UI.MessageBox.show(msg, "更新結果", DQ.UI.MessageBox.Buttons.OK, DQ.UI.MessageBox.Icon.Asterisk);
                }
            },
            error: function (XMLHttpRequest, textStatus, errorThrown) {
                alert(DQ.format("{0}: {1} {2}", XMLHttpRequest.status, XMLHttpRequest.statusText, textStatus));
            }
        });
    }
}

DQ.droppable = function (obj, callback) {
    function init() {
        obj.bind("dragenter", dragenterEventProcess);
        obj.bind("dragover", dragoverEventProcess);
        obj.bind("dragleave", dragleaveEventProcess);
        obj.bind("drop", dropEventProcess);
    }

    function dragenterEventProcess(event) {
        event.preventDefault();
        obj.text("drag entered");
        return false;
    }


    function dragoverEventProcess(event) {
        event.preventDefault();
        obj.text("now on draggin...");
        return false;
    }


    function dragleaveEventProcess(event) {
        event.preventDefault();
        obj.text("leaved.");
        return false;
    }


    function dropEventProcess(event) {
        event.preventDefault();
        var e = event.originalEvent || event;
        console.log(e.dataTransfer.files);
        callback && callback(e.dataTransfer.files);
        console.log("hear");
        if (false) {
            var files = event.originalEvent.dataTransfer.files;
            if (event.originalEvent.dataTransfer.files) {
                var buffer = "You've just dropped " + files.length + " files.<hr />";
                for (var i = 0; i < files.length; i++) {
                    var file = files[i];
                    buffer += "ファイル名: " + file.name + "<br />";
                    buffer += "タイプ: " + file.type + "<br />";
                    buffer += "URN: " + file.urn + "<hr />";
                }
            }
            else {
                var dat = [];
                for (var nm in event.dataTransfer) {
                    dat.push(nm);
                }
                buffer = dat.join("<br/>");
                buffer += event.dataTransfer.getData("URL");
            }

            obj.text(buffer);
        }
        event.stopPropagation();
        event.stopImmediatePropagation();
        return false;
    }

    init();
}

DQ.upload = function (options) {
    options = DQ.options(options, {
        url: "upload.php5",
        dataType: "json",
        data: {},
        fileInput: null,
        callback: null,
        maxSize: 1024 * 1024
    });
    if (!options.fileInput) {
        throw new Error("fileInput options is must specified.");
    }

    var fileInput = options.fileInput;
    var id = new Date().getTime(),
        formId = "form" + id,
        frameId = "frame" + id,
        fileId = "file" + id,
        form = $('<form>').attr({
            id: formId,
            method: "POST",
            name: formId,
            action: options.url,
            target: frameId,
            enctype: 'multipart/form-data'
        }).css({
            'position': 'absolute',
            'top': '-1200px',
            'left': '-1200px'
        }).appendTo(DQ.page().obj),
        frame = $('<iframe>').attr({
            id: frameId,
            name: frameId
        }).css({
            'position': 'absolute',
            'left': '-9999px',
            'top': '-9999px'
        }
    ).appendTo(DQ.page().obj);

    if (window.ActiveXObject) {
        frame.attr('src', 'javascript:false');
    }
    for (var nm in options.data) {
        $('<input>').attr({
            type: 'hidden',
            name: nm,
            value: options.data[nm]
        }).appendTo(form);
    }
    var newElement = fileInput.clone();
    fileInput.attr('id', fileId);
    fileInput.before(newElement);
    fileInput.appendTo(form);
    $('<input>').attr({
        type: 'hidden',
        name: 'MAX_FILE_SIZE',
        value: options.maxSize
    }).before(fileInput);
    var xml = {}
    frame.load(function (e) {
        var io = frame[0];
        try {
            if (io.contentWindow) {
                xml.responseText = io.contentWindow.document.body ? io.contentWindow.document.body.innerHTML : null;
                xml.responseXML = io.contentWindow.document.XMLDocument ? io.contentWindow.document.XMLDocument : io.contentWindow.document;

            } else if (io.contentDocument) {
                xml.responseText = io.contentDocument.document.body ? io.contentDocument.document.body.innerHTML : null;
                xml.responseXML = io.contentDocument.document.XMLDocument ? io.contentDocument.document.XMLDocument : io.contentDocument.document;
            }
        } catch (e) {
            alert(e);
        }
        var res;
        if (options.dataType == "json") {
            eval("res = " + xml.responseText);
        } else {
            res = xml.responseText;
        }
        options.callback && options.callback(res);
        form.remove();
        setTimeout(function () {
            frame.remove();
        }, 100);
    });
    if (form[0].encoding) {
        form.attr('encoding', 'multipart/form-data');
        form[0].encoding = 'multipart/form-data';
    }
    else {
        form.attr('enctype', 'multipart/form-data');
    }
    form.submit();
}
DQ.upload.progress = function (options) {
    DQ.options(options, {
        url: "tmpSize.php5",
        speed: 100,
        callback: null
    });
    _progress();

    function _progress() {
        $.ajax({
            url: options.url,
            success: function (res) {
                callback && callback(res);
                if (DQ.upload.uploading) {
                    setTimeout(_progress, options.speed);
                }
            },
            error: function (XMLHttpRequest, textStatus, errorThrown) {
                alert(DQ.format("{0}: {1} {2}", XMLHttpRequest.status, XMLHttpRequest.statusText, textStatus));
            }
        });
    }
}