﻿/// <reference path="jquery.js" />
/// <reference path="dq.js" />
/// <reference path="dq-core.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0004
*/
DQ.lazyLoad(DQ.subdir + "dq-core.js", "DQ.Window", function () {
    DQ.Screen.MapByCanvas = dqextend(DQ.Control, function (screen, options) {
        /// <summary>
        ///     チップマップを作成します。
        /// </sumamry>
        /// <param name="screen" type="DQ.Screen">
        ///    マップの表示先であるスクリーンを指定
        /// </param>
        /// <param name="url" type="string">
        ///     マップデータを読み込むためのURLを指定
        /// </param>
        /// <param name="options" type="object">
        ///     マップのオプションを指定
        /// </param>

        if (!arguments.length) {
            throw new Error("DQ.Screen.Map.constractor: 引数が少なすぎ");
        }

        var me = this;
        options = DQ.options(options, {
            chipSize: DQ.RESOLUTION,
            npcHeight: 48,
            deltaX: DQ.MAP_DELTA,
            deltaY: DQ.MAP_DELTA,
            sprite: DQ.SPRITE32,
            tile1: "tile32.png",
            sprite16: DQ.SPRITE16
        });

        me._npcHeight = options.npcHeight;
        me.chipSize = options.chipSize;
        me.deltaX = options.delta;
        me.deltaY = options.delta;
        me.sprite = options.sprite;
        me.sprite16 = options.sprite16;
        me.tile1 = options.tile1;

        me._l = -me.chipSize + me.chipSize;
        me._r = -me.chipSize - me.chipSize;
        me._u = -me.chipSize + me.chipSize;
        me._d = -me.chipSize - me.chipSize;

        me.scrolling = false;

        //map上の座標
        me.mapX = 50;
        me.mapY = 50;
        me.location = new DQ.Location(me.mapX, me.mapY);

        //マップの最大幅
        me.mapWidth = 100;
        me.mapHeight = 100;
        //
        me.data = null;
        me.canMove = [];
        me.canMoveT = [];
        //
        me._create(screen, options);

        me.setTop(-me.chipSize);
        me.setLeft(-me.chipSize);

        cur = new Date();

        return me;
    }, {
        dqid: "DQ.Screen.Map",
        _dir: null,
        deltaX: DQ.MAP_DELTA,
        deltaY: DQ.MAP_DELTA,
        defaultRank: 0,
        dungeon: false,
        encount: false,
        dragging: false,
        layer: [],
        harf: false,
        modified: false,
        screen: null,
        sprite: null,
        speed: DQ.SCROLL_SPEED,
        _rx: 0,
        _ry: 0,
        _npcHeight: 48,
        npcHeight: function (value) {
            if (!arguments.length) {
                return this._npcHeight;
            }
            this._npcHeight = value;
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].npcHeight && this.layer[i].npcHeight(this._npcHeight);
            }
        },
        resize: function () {
            //マップ表示領域の幅と高さ
            this.width(this.screen.view.width + this.chipSize * 2);
            this.height(this.screen.view.height + this.chipSize * 2);

            //空のスプライトを配置する
            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth),
                maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);

            if (this.chipSize == 32) {
                this.chipSize = 16;
                this.normal();
            } else {
                this.chipSize = 32;
                this.small();
            }
        },
        _create: function (screen, options) {
            /// <summary>
            /// マップを作成します
            /// </sumamry>

            //マップ用スプライト
            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + ".png", me = this,
                tg = $('<div>')
                .addClass('dq-map-32px')
                .css({ background: 'gray', position: 'relative', top: this.top, left: this.left });

            this.base(screen, tg);

            this.obj0 = this.obj[0];

            me.image = new Image();
            me.image.src = sp;

            this.data = [];
            this.canMove = [];
            this.canMoveT = [];
            this.numberOfLayer = 2;
            if (this.canMove.length == 0) {
                for (var i = 0; i < 64; i++) {
                    this.canMove[i] = 1;
                }
            }
            if (this.canMoveT.length == 0) {
                for (var i = 0; i < 256; i++) {
                    this.canMoveT[i] = 1;
                }
            }
            //マップデータの初期化
            this.layer1 = [];
            for (var y = 0; y < this.mapHeight; y++) {
                this.data.push([]);
                this.layer1.push([]);
                for (var x = 0; x < this.mapWidth; x++) {
                    this.data[y].push(0);
                    this.layer1[y].push(0);
                }
            }

            this.screen = screen;
            this.resize();
            screen.append(tg);

            //マウスドラッグイベントを設定する
            this.chipSize = 16; //ダミー
            this.normal();

            //レイヤーを追加
            this.layer = [];
            this.layer.push(new DQ.Screen.HighLayer(screen, this));
            //this.layer.push(new DQ.Screen.HighLayer(screen, this));

            //マップデータを非同期で受信
            options.url && this.loadMap(options.url);
        },
        _createCanvas: function (mapWidth, mapHeight, chipSize) {
            if (mapWidth > 200 || mapHeight > 200) {
                throw new InvalidArgument("上限は200です");
            }
            this._canvas && this._canvas.remove();
            this._canvas =
                $(document.createElement('canvas')).appendTo(this.obj)
                .css({ position: 'absolute', top: 0, left: 0 })
                .attr({ 'width': mapWidth * chipSize, 'height': mapHeight * chipSize });

            this.ctx = this._canvas[0].getContext("2d");
            this._cvs0 = this._canvas[0];

            this._canvasH && this._canvasH.remove();
            this._canvasH =
                $(document.createElement('canvas')).appendTo(this.obj)
                .css({ position: 'absolute', top: 0, left: 0, 'z-index': 200 })
                .attr({ 'width': mapWidth * chipSize, 'height': mapHeight * chipSize });

            this.ctxH = this._canvasH[0].getContext("2d");
            this._cvsH0 = this._canvasH[0];
        },
        _createCanvasH: function (mapWidth, mapHeight, chipSize) {
            if (mapWidth > 200 || mapHeight > 200) {
                throw new InvalidArgument("上限は200です");
            }
            this._canvasH && this._canvasH.remove();
            this._canvasH =
                $(document.createElement('canvas')).appendTo(this.obj)
                .css({ position: 'absolute', top: 0, left: 0, 'z-index': 200 })
                .attr({ 'width': mapWidth * chipSize, 'height': mapHeight * chipSize });

            this.ctxH = this._canvasH[0].getContext("2d");
            this._cvsH0 = this._canvasH[0];
        },
        small: function () {
            /// <summary>
            ///     マップのチップサイズを16pxへ変更します
            /// </summary>
            if (this.chipSize == 16) {
                return;
            }
            this.chipSize = 16;
            var tg = this.obj.html("").removeClass('dq-map-32px').addClass('dq-map-16px');

            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + ".png";
            if (!this.simage || this.simage.src != pn + "shadow32.png") {
                this.simage = new Image();
                this.simage.src = pn + "shadow32.png";
            }
            if (!this.timage || this.timage.src != pn + this.tile1) {
                this.timage = new Image();
                this.timage.src = pn + this.tile1;
            }

            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);
            this._createCanvas(maxX, maxY, this.chipSize);

            this.setTop(-this.chipSize);
            this.setLeft(-this.chipSize);

            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.dt = [];
            this.dl = [];
            for (var d = 0; d < 64; d++) {
                this.dt[d] = -(d % 8) * this.chipSize + 'px';
                this.dl[d] = -Math.floor(d / 8) * this.chipSize + 'px';
            }
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].small();
            }
            this.shadow = [];
            this.shadow.length = this.maxY * this.maxX;
        },
        normal: function () {
            /// <summary>
            ///     マップのチップサイズを32pxへ変更します
            /// </summary>
            if (this.chipSize == 32) {
                return;
            }
            this.chipSize = 32;


            var pn = world.project ? world.project.name + "/images/" : "images/";
            var sp = pn + this.sprite + ".png";
            if (!this.simage || this.simage.src != pn + "shadow32.png") {
                this.simage = new Image();
                this.simage.src = pn + "shadow32.png";
            }
            if (!this.timage || this.timage.src != pn + this.tile1) {
                this.timage = new Image();
                this.timage.src = pn + this.tile1;
            }

            var maxX = this.maxX = Math.min(Math.floor(this._width / this.chipSize), this.mapWidth);
            var maxY = this.maxY = Math.min(Math.floor(this._height / this.chipSize), this.mapHeight);
            this._createCanvas(maxX, maxY, this.chipSize);

            this.setTop(-this.chipSize);
            this.setLeft(-this.chipSize);

            this._l = -this.chipSize + this.chipSize;
            this._r = -this.chipSize - this.chipSize;
            this._u = -this.chipSize + this.chipSize;
            this._d = -this.chipSize - this.chipSize;

            this.dt = [];
            this.dl = [];
            for (var d = 0; d < 64; d++) {
                this.dt[d] = -(d % 8) * this.chipSize + 'px';
                this.dl[d] = -Math.floor(d / 8) * this.chipSize + 'px';
            }
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].normal();
            }
            this.shadow = [];
            this.shadow.length = this.maxY * this.maxX;
        },
        loading: false,
        loadMap: function (url, to) {
            /// <summary>
            ///     マップデータをサーバーから取得します。
            /// </summary>
            /// <param name="url" type="string">
            ///  マップを読み込むためのURLを指定
            /// </param>
            /// <param name="to" type="array">
            ///  デフォルト以外の初期位置
            /// </param>

            var me = this;
            me.loading = true;
            var hi = this.layer[0];
            var pn = world.project ? world.project.name + "/" : "";
            $.ajax(
            {
                type: "GET",
                url: pn + url,
                cache: false,
                success: function (data) {
                    try {
                        eval('var s=' + data);
                    } catch (ex) {
                        alert(DQ.format("{0}\n{1} : {2}", url, ex.name, ex.message));
                        me.loading = false;
                        return;
                    }
                    me._map = s;
                    me.url = url;

                    me.canMove = s.canMove || [];
                    me.canMoveT = s.canMoveT || [];
                    if (me.canMoveT.length == 0) {
                        for (var i = 0; i < 256; i++) {
                            me.canMoveT[i] = 1;
                        }
                    }
                    me.topographyType = s.topographyType || [
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3,
                        3, 3, 3, 3, 3, 3, 3, 3];
                    me.name = s.name || "";
                    me.uid = s.uid || -1;
                    me.data = s.data;
                    me.numberOfLayer = s.numberOfLayer || 2;
                    me.mapWidth = s.mx || me.mapWidth;
                    me.mapHeight = s.my || me.mapHeight;
                    me.ix = s.ix;
                    me.iy = s.iy;
                    me.mapX = (s.ix != null) ? s.ix - DQ.OX : me.mapX;
                    me.mapY = (s.iy != null) ? s.iy - DQ.OY : me.mapY;
                    me.mapX = (to != null) ? to[0] : me.mapX;
                    me.mapY = (to != null) ? to[1] : me.mapY;
                    if (me.mapX < 0) {
                        me.mapX = 0;
                    }
                    if (me.mapY < 0) {
                        me.mapY = 0;
                    }
                    //me.location = new DQ.Location(me.mapX, me.mapY);

                    me.maxX = Math.min(Math.floor(me._width / me.chipSize), me.mapWidth);
                    me.maxY = Math.min(Math.floor(me._height / me.chipSize), me.mapHeight);

                    me.encount = (s.encount != null) ? s.encount : true;
                    me.dungeon = (s.dungeon != null) ? s.dungeon : false;
                    me.defaultRank = (s.defaultRank != null) ? s.defaultRank : 0;
                    if (s.sprite) {
                        me.sprite = s.sprite;
                    } else {
                        me.sprite = DQ.SPRITE32;
                    }

                    me.setTop(-me.chipSize);
                    me.setLeft(-me.chipSize);

                    me._l = -me.chipSize + me.chipSize;
                    me._r = -me.chipSize - me.chipSize;
                    me._u = -me.chipSize + me.chipSize;
                    me._d = -me.chipSize - me.chipSize;
                    me.location.reset();

                    hi.importSetup(me);
                    hi.data = [];

                    me.onLoaded && me.onLoaded(me);
                    me.loading = false;
                    me._dmax = 0;

                    me.scrolling = false;

                    for (var i = 1; i < me.numberOfLayer; i++) {
                        var nm = "layer" + i;
                        if (!s[nm]) {
                            s[nm] = [];
                            var sl = s[nm];
                            for (var y = 0; y < me.mapHeight; y++) {
                                sl.push([]);
                                for (var x = 0; x < me.mapWidth; x++) {
                                    sl[y].push(0);
                                }
                            }
                        }
                        me[nm] = s[nm];
                    }

                    var sp = pn + "images/" + me.sprite + ".png";
                    me.image = new Image();
                    $(me.image).one('load', function () {
                        me.resize();
                        me.forceMap();
                    });
                    me.image.src = sp;

                    //マップの読み込み途中をマスクするレイヤーを削除する（マスクされていれば)。
                    me.screen.ly && me.screen.ly.dispose();
                    me.modified = false;
                },
                error: function (XMLHttpRequest, textStatus, errorThrown) {
                    alert(DQ.format("{0}\n{1}: {2} {3}", pn + url, XMLHttpRequest.status, XMLHttpRequest.statusText, textStatus));
                    me.loading = false;
                }
            });
        },
        forceMap: function _map$forceMap() {
            var maxY = this.maxY,
                maxX = this.maxX;

            if (this.mapX + maxX >= this.mapWidth) {
                this.mapX = this.mapWidth - maxX;
            }
            if (this.mapY + maxY >= this.mapHeight) {
                this.mapY = this.mapHeight - maxY;

            }
            if (this.mapX < 0) {
                this.mapX = 0;
            }
            if (this.mapY < 0) {
                this.mapY = 0;
            }

            var mapX = this.mapX,
                mapY = this.mapY;
            var cx = this.mapX,
                cy = this.mapY,
                sz = this.chipSize,
                ctx = this.ctx,
                ctxH = this.ctxH,
                sprite = this.image, me = this,
                shadow = this.simage,
                tile1 = this.timage,
                canMove = this.canMove,
                canMoveT = this.canMoveT;

            this._cd = 0;
            ctx.clearRect(0, 0, this._canvas.width(), this._canvas.height());
            ctxH.clearRect(0, 0, this._canvas.width(), this._canvas.height());
            for (var y = 0; y < maxY; y++) {
                var dy = this.data[mapY + y],
                    ly = this.layer1[mapY + y],
                    dly = this.data[mapY + y - 1];
                for (var x = 0; x < maxX; x++) {
                    var d = dy[cx + x],
                        l = ly[cx + x],
                        dd = dy[mapX + x - 1],
                        ll = dly == null ? 0 : dly[mapX + x - 1],
                        sy = (d % 8) * 32,     //この8 は 256px / 32px より算出
                        sx = Math.floor(d / 8) * 32;


                    if (d == 33 || d == 36 || d == 39 || d == 49 || d == 52 || d == 55) {
                        ctx.drawImage(sprite, sx + 16, sy + 16, 32, 32, x * sz, y * sz, sz, sz);
                    }
                    else {
                        ctx.drawImage(sprite, sx, sy, 32, 32, x * sz, y * sz, sz, sz);
                    }
                    if (d == 32 || d == 35 || d == 38 || d == 48 || d == 51 || d == 54) {
                        this.autoTile(x + mapX, y + mapY, (d - 32));
                    }
                    if (l != 0) {
                        if (canMoveT[l] == 2) {
                            ctxH.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                        } else {
                            ctx.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                        }
                    }

                    //auto shadow.
                    if (canMove[d] && (dd == 13 || dd == 14)) {
                        if (canMove[ll]) {
                            ctx.drawImage(shadow, 0, 32, 32, 32, x * sz, y * sz, sz, sz);
                        } else {
                            ctx.drawImage(shadow, 0, 64, 32, 32, x * sz, y * sz, sz, sz);
                        }
                    }
                }
            }
        },
        _viewMap: function (ox, oy, view, data, maxX, maxY, shadow, sz, tile1) {
            data = this.data;
            view = this.view;
            maxY = this.maxY;
            maxX = this.maxX;
            sz = this.chipSize;
            shadow = this.simage;
            tile1 = this.timage;

            var ctx = this.ctx,
                ctxH = this.ctxH,
                sprite = this.image;
            ctxH.clearRect(0, 0, maxX * sz, maxY * sz);
            for (var y = 0; y < maxY; y++) {
                var my = y * maxX,
                    cy = this.mapY + y,
                    cd = data[cy + oy],
                    dy = data[cy],
                    ly = this.layer1[cy],
                    dly = data[this.mapY + y - 1];
                for (var x = 0; x < maxX; x++) {
                    var cx = this.mapX + x;
                    var d = dy[cx],
                        l = ly[cx],
                        dd = dy[this.mapX + x - 1],
                        sy = (d % 8) * 32,     //この8 は 256px / 32px より算出
                        sx = Math.floor(d / 8) * 32;

                    if (d == 32 || d == 35 || d == 38 || d == 48 || d == 51 || d == 54) {
                        this.autoTile(x + this.mapX, y + this.mapY, (d - 32));
                    } else {
                        if (d == 33 || d == 36 || d == 39 || d == 49 || d == 52 || d == 55) {
                            ctx.drawImage(sprite, sx + 16, sy + 16, 32, 32, x * sz, y * sz, sz, sz);
                        }
                        else {
                            ctx.drawImage(sprite, sx, sy, 32, 32, x * sz, y * sz, sz, sz);
                        }
                    }
                    if (l != 0) {
                        if (this.canMoveT[l] == 2) {
                            ctxH.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                        } else {
                            ctx.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                        }
                    }
                    //コストを最小にするため変化が無ければ書き換えない
                    //                    if (d != this.data[cy + oy][cx + ox] || (dd == 13 || dd == 15)) {
                    //                    }
                    var ll = !dly ? 0 : dly[cx - 1];
                    if (this.canMove[d] && (dd == 13 || dd == 14)) {
                        if (this.canMove[ll]) {
                            ctx.drawImage(shadow, 0, 32, 32, 32, x * sz, y * sz, sz, sz);
                        } else {
                            ctx.drawImage(shadow, 0, 64, 32, 32, x * sz, y * sz, sz, sz);
                        }
                    }
                }
            }
        },
        move: function (x, y) {

            this.mapX = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            this.mapY = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = this.mapX >= 0 ? this.mapX : 0;
            this.mapY = this.mapY >= 0 ? this.mapY : 0;
            this.forceMap();
            for (var i = 0; i < this.layer.length; i++) {
                this.layer[i].mapX = this.mapX;
                this.layer[i].mapY = this.mapY;
                this.layer[i].forceMap();
            }

        },
        skip: function (ox, oy) {
            /// <summary>
            /// 指定のオフセット分現在地を移動します。
            /// </summary>
            var x = this.mapX + ox;
            var y = this.mapY + oy;

            var px = this.mapX;
            var py = this.mapY;
            x = x < this.mapWidth - this.maxX ? x : this.mapWidth - this.maxX - 1;
            y = y < this.mapHeight - this.maxY ? y : this.mapHeight - this.maxY - 1;
            this.mapX = x >= 0 ? x : 0;
            this.mapY = y >= 0 ? y : 0;

            var dx = this.mapX - px;
            var dy = this.mapY - py;
            this._viewMap(-dx, -dy);

            this.onScrolled && this.onScrolled(this, null) == false && (this._dir = null);
        },
        setLeft: function (value) {
            this.left = value;
            this.obj0.style.left = value + 'px';
        },
        setTop: function (value) {
            this.top = value;
            this.obj0.style.top = value + 'px';
        },
        autoTile: function (x, y, no) {
            var d = this.data[y][x];
            var sz = this.chipSize,
                csz = this.chipSize / 2;
            var set = [
                this.data[y - 1][x - 1] == d || this.data[y - 1][x - 1] == d + 1,
                this.data[y - 1][x] == d || this.data[y - 1][x] == d + 1,
                this.data[y - 1][x + 1] == d || this.data[y - 1][x + 1] == d + 1,
                this.data[y][x - 1] == d || this.data[y][x - 1] == d + 1,
                this.data[y][x],
                this.data[y][x + 1] == d || this.data[y][x + 1] == d + 1,
                this.data[y + 1][x - 1] == d || this.data[y + 1][x - 1] == d + 1,
                this.data[y + 1][x] == d || this.data[y + 1][x] == d + 1,
                this.data[y + 1][x + 1] == d || this.data[y + 1][x + 1] == d + 1
                ];
            x -= this.mapX;
            y -= this.mapY;

            var of;
            //左上
            if (!set[3] && !set[1]) {
                //角
                of = { top: 2 * 16, left: 0 };
            } else if (!set[3]) {
                //左
                of = { top: 3 * 16, left: 0 };
            } else if (!set[1]) {
                //上
                of = { top: 2 * 16, left: 2 * 16 };
            } else if (!set[0]) {
                //外角
                of = { top: 0, left: 2 * 16 };
            } else {
                //普通
                of = { top: 3 * 16, left: 1 * 16 };
            }
            var tt = (no % 8) * 32;
            var ll = (Math.floor(no / 8) + 4) * 32;
            this.ctx.drawImage(
                this.image,
                ll + of.left, tt + of.top, 16, 16,
                x * sz + 0 * 16, y * sz + 0 * csz, csz, csz);
            //左下
            if (!set[3] && !set[7]) {
                //角
                of = { top: 5 * 16, left: 0 };
            } else if (!set[3]) {
                //左
                of = { top: 4 * 16, left: 0 };
            } else if (!set[7]) {
                //下
                of = { top: 5 * 16, left: 2 * 16 };
            } else if (!set[6]) {
                //外角
                of = { top: 1 * 16, left: 2 * 16 };
            } else {
                //普通
                of = { top: 4 * 16, left: 1 * 16 };
            }
            this.ctx.drawImage(
                this.image,
                ll + of.left, tt + of.top, 16, 16,
                x * sz + 0 * 16, y * sz + 1 * csz, csz, csz);

            //右上
            if (!set[5] && !set[1]) {
                //角
                of = { top: 2 * 16, left: 3 * 16 };
            } else if (!set[5]) {
                //右
                of = { top: 3 * 16, left: 3 * 16 };
            } else if (!set[1]) {
                //上
                of = { top: 2 * 16, left: 2 * 16 };
            } else if (!set[2]) {
                //外角
                of = { top: 0, left: 3 * 16 };
            } else {
                //普通
                of = { top: 3 * 16, left: 2 * 16 };
            }
            this.ctx.drawImage(
                this.image,
                ll + of.left, tt + of.top, 16, 16,
                x * sz + 1 * csz, y * sz + 0 * csz, csz, csz);

            //右下
            if (!set[5] && !set[7]) {
                //角
                of = { top: 5 * 16, left: 3 * 16 };
            } else if (!set[5]) {
                //右
                of = { top: 4 * 16, left: 3 * 16 };
            } else if (!set[7]) {
                //下
                of = { top: 5 * 16, left: 2 * 16 };
            } else if (!set[8]) {
                //外角
                of = { top: 1 * 16, left: 3 * 16 };
            } else {
                //普通
                of = { top: 4 * 16, left: 2 * 16 };
            }
            this.ctx.drawImage(
                this.image,
                ll + of.left, tt + of.top, 16, 16,
                x * sz + 1 * csz, y * sz + 1 * csz, csz, csz);
        },
        scrollUp: function () {
            this.top += this.deltaY;
            if (this.top >= this._u) {
                this.mapY--;
                this._viewMap(0, 1);
                this.top = -this.chipSize;
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.up();
                return false;
            } else if (this.top == this._u - this.chipSize / 2 || this.top == this._d + this.chipSize) {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.up();
                return false;
            } else {
                this.obj0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollDown: function () {
            this.top -= this.deltaY;
            if (this.top < this._d) {
                this.top = -this.chipSize;
                this.mapY++;
                this._viewMap(0, -1);
                this.obj0.style.top = this.top + 'px';
                //this._cvs0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.down();
                return false;
            } else if (this.top == this._d + this.chipSize / 2 || this.top == this._d + this.chipSize) {
                this.obj0.style.top = this.top + 'px';
                //this._cvs0.style.top = this.top + 'px';
                this.scrolling = false;
                this.location.down();
                return false;
            } else {
                this.obj0.style.top = this.top + 'px';
                //this._cvs0.style.top = this.top + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollLeft: function () {
            this.left += this.deltaX;
            if (this.left >= this._l) {
                this.left = -this.chipSize;
                this.mapX--;
                this._viewMap(1, 0);
                this.obj0.style.left = this.left + 'px';
                //this._cvs0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.left();
                return false;
            } else if (this.left == this._l - this.chipSize / 2 || this.left == this._r + this.chipSize) {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.left();
                return false;
            } else {
                this.obj0.style.left = this.left + 'px';
                //this._cvs0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scrollRight: function () {
            this.left -= this.deltaX;
            if (this.left < this._r) {
                this.left = -this.chipSize;
                this.mapX++;
                this._viewMap(-1, 0);
                this.obj0.style.left = this.left + 'px';
                //this._cvs0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.right();
                return false;
            } else if (this.left == this._r + this.chipSize / 2 || this.left == this._r + this.chipSize) {
                this.obj0.style.left = this.left + 'px';
                this.scrolling = false;
                this.location.right();
                return false;
            } else {
                this.obj0.style.left = this.left + 'px';
                //this._cvs0.style.left = this.left + 'px';
                this.scrolling = true;
                return true;
            }
        },
        scroll: function (dir) {
            ///<summary>
            /// 指定の方向へスクロール(歩く)します。
            ///</summary>
            ///<param name="dir" type="string">
            /// 方向を指定(left,right,up,down)
            ///</param>

            switch (dir) {
                case DQ.DIR.UP:
                    if (this.mapY <= 0) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.DOWN:
                    if (this.mapY >= this.mapHeight - this.maxY - 1) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.LEFT:
                    if (this.mapX <= 0) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                case DQ.DIR.RIGHT:
                    if (this.mapX >= this.mapWidth - this.maxX - 1) {
                        this._dir = null;
                        this.onBorder && this.onBorder(this, dir);
                        return false;
                    }
                    break;
                default:
                    this._dir = null;
                    return false;
            }

            //次回スクロール予約
            this._dir = dir;
            !this.scrolling && this.location.makeTarget(dir);

            return true;
        },
        stop: function () {
            this._dir = null;
        },
        _keep: null,
        _dmax: 0,
        _cd: 0,
        _aniPos: 0,
        animation: function _map$animation() {

            this._cd++;
            if (this._cd < this.frame) {
                return;
            }
            this._cd = 0;
            var apos = this._aniPos = this._aniPos ? 0 : 32;

            var maxY = this.maxY,
                maxX = this.maxX;
            var cx = this.mapX,
                cy = this.mapY,
                sz = this.chipSize,
                ctx = this.ctx,
                ctxH = this.ctxH,
                sprite = this.image, me = this,
                shadow = this.simage,
                tile1 = this.timage;
            for (var y = 0; y < maxY; y++) {
                var dy = this.data[this.mapY + y],
                    ly = this.layer1[this.mapY + y],
                    dly = this.data[this.mapY + y - 1];
                for (var x = 0; x < maxX; x++) {
                    var d = dy[cx + x],
                        l = ly[cx + x],
                        dd = dy[this.mapX + x - 1],
                        ll = dly == null ? 0 : dly[this.mapX + x - 1],
                        sy = (d % 8) * 32,     //この8 は 256px / 32px より算出
                        sx = Math.floor(d / 8) * 32;
                    if (d == 24 || d == 27 || d == 30) {
                        ctx.drawImage(sprite, sx, sy + apos, 32, 32, x * sz, y * sz, sz, sz);

                        if (this.canMove[d] && (dd == 13 || dd == 14)) {
                            if (this.canMove[ll]) {
                                ctx.drawImage(shadow, 0, 32, 32, 32, x * sz, y * sz, sz, sz);
                            } else {
                                ctx.drawImage(shadow, 0, 64, 32, 32, x * sz, y * sz, sz, sz);
                            }
                        }
                        if (l != 0) {
                            if (this.canMoveT[l] == 2) {
                                ctxH.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                            } else {
                                ctx.drawImage(tile1, (l % 16) * 32, Math.floor(l / 16) * 32, 32, 32, x * sz, y * sz, sz, sz);
                            }
                        }
                    }
                }
            }
        },
        update: function _map$update() {
            if (!this.scrolling) {
                this.animation();
                this.layer[0].walk();
            }
            end = new Date();
            $('#ms').text(end - cur);
            cur = new Date();

            var me = this;
            !this.loading && _scroll();
            var dir = null;

            function _scroll() {
                var delta = { '0': [0, -1], '1': [0, 1], '2': [-1, 0], '3': [1, 0] };
                if (!me._keep && me.onPrescroll && me.scroll(me._dir)) {
                    if (!me.scrolling && me.onPrescroll(me, dir, delta[me._dir])) {
                        me._dir = null;
                        return false;
                    }
                }

                dir = !me._keep ? me._dir : me._keep;
                if (dir == null) {
                    //次がなければ何もしない
                    return false;
                }
                var res;
                switch (dir) {
                    case DQ.DIR.UP:
                        res = me.scrollUp();
                        break;
                    case DQ.DIR.DOWN:
                        res = me.scrollDown();
                        break;
                    case DQ.DIR.LEFT:
                        res = me.scrollLeft();
                        break;
                    case DQ.DIR.RIGHT:
                        res = me.scrollRight();
                        break;
                }

                var hi = me.layer[0];
                hi && hi.scroll(dir);
                if (res) {
                    me.onScrolling && me.onScrolling(me, dir);
                    me._keep = dir;
                } else {
                    this._dir = me.onScrolled && me.onScrolled(me, dir) == false ? null : this._dir;
                    me._keep = null;
                }
                return res;
            }
        },
        onClick: null,
        onLoaded: null,
        onBorder: null,
        onPrescroll: null,
        onScrolling: null,
        onScrolled: null,
        _click: function (e) {
            var me = e.data;
            var x = Math.floor(me.screen.toClientX(e.clientX) / me.chipSize);
            var y = Math.floor(me.screen.toClientY(e.clientY) / me.chipSize);
            me.onClick && me.onClick.apply(me, [e, { left: x, top: y}]);
        },
        click: function (fn) {
            if (!fn) {
                this.obj.unbind("click", this._click);
            } else {
                this.onClick = fn;
                this.obj.bind("click", this, this._click);
            }
        },
        scrolled: function (fn) {
            this.onScrolled = fn;
        },
        prescroll: function (fn) {
            this.onPrescroll = fn;
        },
        keydown: function (fn) {
            this.parent._keydown = fn;
        },
        getNPC: function () {
            return this.layer[0].npc;
        }
    });

});
