﻿/// <reference path="../../js/jquery.js" />
/// <reference path="../../js/dq.js" />
/// <reference path="../../js/dq-core.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0003
*/

DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg-item.js", "DQ.RPG.ItemCatalog");
DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg-creature.js", "DQ.RPG.Creature", function () {
    DQ.RPG.Weapon = function (options) {
        options = DQ.options(options, {
            uid: 0,
            name: "unk",
            text: "不明",
            description: "不明",
            category: "",
            type: "weapon",
            STR: 0,
            VIT: 0,
            AGI: 0,
            ATK: 0,
            DEF: 0,
            HP: 0,
            MP: 0,
            count: 1,
            Buy: 0,
            Sell: 0,
            state: {},
            property: DQ.RPG.DamageProperty.Pysical,
            img: "data/item_default.png"
        });
        this._create(options);
        this._strategy = this._buildStrategy(options);
    }
    DQ.RPG.Weapon.table = [];
    DQ.RPG.Weapon.prototype = {
        _buildStrategy: function (state) {
            //特殊な攻撃名はとりあえず無い
            for (var i = 0; i < DQ.RPG.Weapon.table.length; i++) {
                if (DQ.RPG.Weapon.table[i].type == options.type ||
                    DQ.RPG.Weapon.table[i].uid == options.uid) {
                    return new DQ.RPG.Magic.table[i].strategy(this);
                }
            }
            return new DQ.RPG.WeaponStg(this);
        },
        _create: function (options) {
            var c = DQ.RPG.ItemCatalog;
            for (var nm in options) {
                if (!isNaN(options[nm])) {
                    this[nm] = parseInt(options[nm]);
                } else {
                    this[nm] = options[nm];
                }
            }
            this["text"] = options.name;
            if (options.Buy == null) {
                this["Buy"] = options.G;
            }
        },
        clone: function () {
            var guid = new DQ.Guid();
            var c = new DQ.RPG.Weapon();
            c._create(this);
            c.serial = guid.generate();

            return c;
        },
        equip: function (to) {
            return this._strategy.equip(to);
        },
        remove: function (from) {
            return this._strategy.remove(from);
        },
        canUse: function () {
            return this._strategy.canUse();
        },
        _state: null,
        state: function (value) {
            if (!arguments.length) {
                return this._state;
            }
            if (value instanceof DQ.RPG.State) {
                this._state = value;
            } else {
                this._state = new DQ.RPG.State(value);
            }
        },
        attack: function (from, to, text) {
            text.push(DQ.format("{0} は、{1} を攻撃した。", from.name, to.name));
            return this._strategy.attack(from, to, text);
        }
    }
    DQ.RPG.WeaponStg = function (owner) {
        this.owner = owner;
    }
    DQ.RPG.WeaponStg.prototype = {
        canUse: function () {
            return false;
        },
        getDamage: function (from, to) {
            /// <summary>
            /// fromからtoへのダメージを算出します。
            /// </summary>

            //アルゴリズムをAPIへ集約
            var dmg = world.getDamage(from, to);
            return to.gainDamage(this.owner.property, dmg);
        },
        equip: function (to) {
        },
        remove: function (from) {
        },
        attack: function (from, to, text) {
            /// <summary>
            /// 攻撃する
            /// </summary>
            var critical = from.isCriticalHit(to);
            var isHit = from.isHit(to);
            if (!isHit && !critical) {
                text.push(DQ.format("ミス!! \n{0} に、ダメージを与えられない。", to.name));
                return false;
            }
            if (to.isAvoid(from, DQ.RPG.DamageProperty.Pysical)) {
                text.push(DQ.format("{0}は、攻撃をひらりとかわした。", to.name));
                return false;
            }

            var cmd;
            if (isHit && critical) {
                dmg = world.getCriticalDamage(from, to);
            } else {
                dmg = this.getDamage(from, to);
            }
            to.hp(to.hp() < dmg ? 0 : to.hp() - dmg);
            if (dmg == 0) {
                text.push(from.name + "の、攻撃は効かない。");
            } else {
                isHit && critical && text.push(from.name + "は、会心の一撃を与えた。");
                text.push(DQ.format("{0}ポイントのダメージを与えた!!", dmg));
            }
            return true;
        },
        update: function () {
        }
    }
});
