﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0005
*/
DQ.Chara = function (screen, options) {
    /// <summary>
    ///     キャラクター表示
    /// </summary>

    options = DQ.options(options, {
        cls: "dq-chara-48",
        uid: -1,
        image: null,
        xoffset: 32,
        yoffset: 48,
        top: (DQ.OY - 1) * 32 - 16,
        left: (DQ.OX - 1) * 32,
        frame: 8
    });

    this._create(screen, options);
}

DQ.Chara.prototype = {
    cls: "dq-chara-48",
    layer: null,
    pose: 0,
    frame: 8,
    xoffset: 32,
    yoffset: 48,
    _create: function (screen, options) {
        this.cls = "dq-chara-" + options.yoffset;
        var layer = this.layer = new DQ.Screen.Layer(screen, "dq-chara"),
            b = $('<div>').appendTo(layer.obj).addClass(this.cls);

        //イメージ名の決定
        var pn = world.project ? world.project.name + "/" : "";
        if (!options.image) {
            if (options.uid == -1) {
                options.image = pn + "images/chara_" + options.yoffset + ".png";
            } else {
                options.image = pn + "images/chara_" + options.uid + "_" + options.yoffset + ".png";
            }
        }
        this._uid = options.uid;
        this.frame = options.frame;
        this.xoffset = options.xoffset;
        this.yoffset = options.yoffset;
        this.layer.obj.css({ top: options.top, left: options.left, 'z-index': 100 + DQ.OX });
        this.image = b.css('background-image', 'url(' + options.image + ')');
        this._style = this.image[0].style;
    },
    _cd: 0,
    _uid: -1,
    uid: function (uid) {
        if (!arguments.length) {
            return this._uid;
        }
        this._uid = uid;
        this.changeImage(uid);
    },
    changeImage: function (image) {
        var pn = world.project ? world.project.name + "/" : "";
        if (typeof image == "number") {
            this._uid = image;
            if (this._uid == -1) {
                image = "images/chara_" + this.yoffset + ".png";
            } else {
                image = "images/chara_" + this._uid + "_" + this.yoffset + ".png";
            }
        }
        this.image.css('background-image', 'url(' + pn + image + ')');
    },
    height: function (value) {
        if (!arguments.length) {
            return this.yoffset;
        }
        if (value != 16 && value != 24 && value != 32 && value != 48) {
            throw InvalidArgument("DQ.Chara.height: invalid height.");
        }
        this.image.removeClass(this.cls);
        this.yoffset = value;
        this.cls = 'dq-chara-' + this.yoffset;
        this.image.addClass(this.cls);
        this.layer.obj.css('z-index', 100 + DQ.OY - 1);

        var top = (DQ.OY - 1) * this.xoffset - ((value == 48 || value == 24) ? (value - this.xoffset) : 0);
        this.layer.css('top', top);
        this.changeImage(this._uid);
    },
    wark: function (dir, force) {
        /// <summary>
        /// 移動してるように見せる処理
        /// </summary>
        /// <param name="dir" type="DQ.DIR">
        /// 移動方法
        /// </param>
        if (this.dir == dir) {
            this._cd++;
            if (this._cd < this.frame && !force) {
                return;
            }
            this._cd = 0;
        }
        this.dir = dir;

        var top = 0, left = 0;
        switch (dir) {
            case DQ.DIR.LEFT:
                top = -1 * this.yoffset;
                break;
            case DQ.DIR.RIGHT:
                top = -2 * this.yoffset;
                break;
            case DQ.DIR.UP:
                top = -3 * this.yoffset;
                break;
            case DQ.DIR.DOWN:
                top = -0 * this.yoffset;
                break;
            default:
                if (this._style.backgroundPosition.length != 0) {
                    top = parseInt(this._style.backgroundPosition.split(" ")[1]);
                }
                break;

        }
        switch (this.pose) {
            case 0:
                left = -0 * this.xoffset;
                this.pose = 1;
                break;
            case 1:
                left = -2 * this.xoffset;
                this.pose = 2;
                break;
            case 2:
                left = -0 * this.xoffset;
                this.pose = 1;
                break;
        }

        this._style.backgroundPosition = left + "px " + top + "px";
    },
    stand: function () {
        this._cd = this.delta;

        var pos = this._style.backgroundPosition;
        pos = pos == "" ? "0px 0px" : pos;

        if (this._style.backgroundPositionX) {
            this._style.backgroundPositionX = '-32px';
        } else {
            debug.writeln("pos:" + pos);
            this._style.backgroundPosition = '-32px ' + pos.split(' ')[1];
        }

        this.pose = 0;
    },
    show: function () {
        this.layer.show();
    },
    hide: function () {
        this.layer.hide();
    }
}

DQ.NPC = function (options) {
    options = DQ.options(options, {
        AI: DQ.Event.AI.Stamp,
        dir: 0,
        image: null,
        pos: [0, 0],
        data: {},
        yoffset: DQ.NPC.yoffset,
        deltaY: world.map.layer[0]._npcOffset
    });
    if (options.image == null) {
        throw new InvalidArgument("DQ.NPC.constracter:: imageオプションは必須");
    }
    this.image(options.image);

    this.AI = options.AI;
    this.dir = options.dir;
    this.event = options.event;
    this.pos = options.pos;
    this.deltaY = options.deltaY;
    this.yoffset = options.yoffset;
    this.top = options.pos[1] * world.map.chipSize - this.deltaY;  //world.map.layer[0]._npcOffset; // -this.event.deltaY;
    this.left = options.pos[0] * world.map.chipSize; // -this.event.deltaX;

    this.location = new DQ.Location(world.map.mapX + this.pos[0], world.map.mapY + this.pos[1]);
}
DQ.NPC.pose = 1;
DQ.NPC.xoffset = 32;
DQ.NPC.yoffset = 48;
DQ.NPC._cd = 0;
DQ.NPC._frame = 0;
DQ.NPC.tick = function () {
    DQ.NPC._cd++;
    if (DQ.NPC._cd < DQ.NPC._frame) {
        return;
    }
    DQ.NPC._cd = 0;
    switch (this.pose) {
        case 0:
            DQ.NPC.pose = 1;
            break;
        case 1:
            DQ.NPC.pose = 2;
            break;
        case 2:
            DQ.NPC.pose = 1;
            break;
    }
}

DQ.NPC.prototype = {
    X: 0,
    Y: 0,
    dir: 0,
    AI: 0,
    yoffset: 48,
    deltaY: 16,
    location: null,
    event: null,
    _imate: null,
    image: function (value) {
        if (arguments.length == 0) {
            return this._imate;
        }
        this._image = value;
        this.obj0 = value[0];
        this.obj0.style.top = this.top + 'px';
        this.obj0.style.left = this.left + 'px';
    },
    position: function (x, y) {
        this.top += (y - this.pos[1]) * world.map.chipSize;
        this.left += (x - this.pos[0]) * world.map.chipSize;
        this.pos = [x, y];
    },
    obj: null,
    warking: false,
    wark: function () {
        ///<summary>
        ///歩けないので、同じ場所でぐるぐる
        //</summary>
        if (this.warking) {
            this.move();
            return;
        }
        if (DQ.NPC._cd != 0) {
            return;
        }
        var cr = [
                { l: 0, g: 128 },
                { l: 128, g: 160 },
                { l: 160, g: 256}];
        var ac = DQ.Random.getValue(cr);
        if (ac == 0) {
            return;
        }

        //歩く
        var ai = DQ.Event.AI;
        if (ac == 1) {
            //同一方向確定
            if (this.AI == ai.None || this.AI == ai.Stamp) {
                return;
            }
        } else {
            //向き変更
            var _LR = function () {
            },
                _TD = function () {
                },
                _ALL = function () {
                    var range = [
                        { l: 0, g: 64 },
                        { l: 64, g: 128 },
                        { l: 128, g: 192 },
                        { l: 192, g: 256}];
                    return DQ.Random.getValue(range) + 1;
                }

            switch (this.AI) {
                case ai.None:
                case ai.Stamp:
                    return;
                case ai.LeftRight:
                case ai.TopDown:
                case ai.Wark:
                    var value = _ALL();
                    if (value != this.dir) {
                        this.event.data.dir = this.dir = value;
                        return;
                    }
                    break;
            }
        }

        this.moveTo(this.dir);
    },
    moveTo: function (dir, callback) {
        this.location.makeTarget(dir);
        var target = this.location.target,
            cancel = false,
            map = world.map;

        for (var i = 0; i < target.length; i++) {
            var xx = this.pos[0] + target[i][0] + map.mapX,
                yy = this.pos[1] + target[i][1] + map.mapY;

            if (world.events.isPreEvent([xx, yy])) {
                cancel = true;
                break;
            }
            if (xx == map.mapX + DQ.OX && yy == map.mapY + DQ.OY) {
                cancel = true;
                break;
            }
            if (world.map.data[yy] == undefined) {
                cancel = true;
                break;
            }
            var p = map.data[yy][xx];
            var h = map.layer1[yy][xx];
            cancel = p == undefined || map.canMove[p] == undefined || !map.canMove[p] || map.canMoveT[h] == 0 || cancel;
        }
        if (cancel) {
            var x = 0;
        } else if (!cancel) {
            this.onMoved = callback;
            this.dir = dir;
            this.move();
        }

        return !cancel;
    },
    moveDown: function () {
        var map = world.map;
        this.top += map.deltaY / 2;
        var delta = this.event.deltaY = this.top - this.pos[1] * map.chipSize + map.layer[0]._npcOffset;
        if (delta >= 32) {
            this.pos[1]++;
            this.obj0.style.top = this.top + 'px';
            this.obj0.style.zIndex = this.pos[1] + 100;
            this.warking = false;
            this.location.down();
            return false;
        } else if (delta == 32 - map.chipSize / 2 || delta == 32 + map.chipSize) {
            this.obj0.style.top = this.top + 'px';
            //this.warking = false;
            this.location.down();
            return true;
        } else {
            this.obj0.style.top = this.top + 'px';
            this.warking = true;
            return true;
        }
    },
    moveRight: function () {
        var map = world.map;
        this.left += map.deltaX / 2;
        var delta = this.event.deltaX = this.left - this.pos[0] * map.chipSize;
        if (delta >= 32) {
            this.pos[0]++;
            this.obj0.style.left = this.left + 'px';
            this.warking = false;
            this.location.right();
            return false;
        } else if (delta == 32 - map.chipSize / 2 || delta == 32 + map.chipSize) {
            this.obj0.style.left = this.left + 'px';
            //this.warking = false;
            this.location.right();
            return true;
        } else {
            this.obj0.style.left = this.left + 'px';
            this.warking = true;
            return true;
        }
    },
    moveUp: function () {
        var map = world.map;
        this.top -= map.deltaY / 2;
        var delta = this.event.deltaY = this.top - this.pos[1] * map.chipSize + map.layer[0]._npcOffset;
        if (delta == -32) {
            this.pos[1]--;
            this.obj0.style.top = this.top + 'px';
            this.obj0.style.zIndex = this.pos[1] + 100;
            this.warking = false;
            this.location.up();
            return false;
        } else if (delta == -32 + map.chipSize / 2 || delta == -32 + map.chipSize) {
            this.obj0.style.top = this.top + 'px';
            //this.warking = false;
            this.location.up();
            return true;
        } else {
            this.obj0.style.top = this.top + 'px';
            this.warking = true;
            return true;
        }
    },
    moveLeft: function () {
        var map = world.map;
        this.left -= map.deltaX / 2;
        var delta = this.event.deltaX = this.left - this.pos[0] * map.chipSize;
        if (delta <= -32) {
            this.pos[0]--;
            this.obj0.style.left = this.left + 'px';
            this.warking = false;
            this.location.left();
            return false;
        } else if (delta == -32 + map.chipSize / 2 || delta == -32 + map.chipSize) {
            this.obj0.style.left = this.left + 'px';
            //this.warking = false;
            this.location.left();
            return true;
        } else {
            this.obj0.style.left = this.left + 'px';
            this.warking = true;
            return true;
        }
    },
    move: function () {
        var pre = [this.pos[0], this.pos[1]], res;
        switch (this.dir) {
            case DQ.DIR.UP:
                res = this.moveUp();
                break;
            case DQ.DIR.DOWN:
                res = this.moveDown();
                break;
            case DQ.DIR.LEFT:
                res = this.moveLeft();
                break;
            case DQ.DIR.RIGHT:
                res = this.moveRight();
                break;
        }
        if (!res) {
            this.event.position = [this.pos[0] + world.map.mapX, this.pos[1] + world.map.mapY];
            this.onMoved && this.onMoved();
            this.onMoved = null;
        }

    },
    direct: function (dir) {
        /// <summary>
        /// 移動してるように見せる処理(向きには未対応)
        /// </summary>
        /// <param name="dir" type="DQ.DIR">
        /// 移動方法
        /// </param>
        if (arguments.length == 0) {
            throw new Error("DQ.NPC.direct:: dir must specified.");
        }
        var obj = this.obj0;
        var top = 0, left;
        switch (dir) {
            case DQ.DIR.LEFT:
                top = -1 * this.yoffset;
                break;
            case DQ.DIR.RIGHT:
                top = -2 * this.yoffset;
                break;
            case DQ.DIR.UP:
                top = -3 * this.yoffset;
                break;
            case DQ.DIR.DOWN:
                top = -0 * this.yoffset;
                break;
            default:
                if (obj.style.backgroundPosition.length != 0) {
                    top = parseInt(obj.style.backgroundPosition.split(" ")[1]);
                }
                break;

        }
        left = -2 * DQ.NPC.xoffset;
        this.dir = dir;
        this.event.data.dir = dir;
        obj.style.backgroundPosition = left + "px " + top + "px";
    },
    stamp: function (dir, force) {
        /// <summary>
        /// 移動してるように見せる処理(向きには未対応)
        /// </summary>
        /// <param name="dir" type="DQ.DIR">
        /// 移動方法
        /// </param>
        if (arguments.length == 0) {
            dir = this.dir;
            force = false;
        }
        if (arguments.length == 1 && typeof dir == "boolean") {
            force = dir;
            dir = this.dir;
        }
        if (DQ.NPC._cd != 0 && !force) {
            return;
        }
        var obj = this.obj0;
        var top = 0, left;
        switch (dir) {
            case DQ.DIR.LEFT:
                top = -1 * this.yoffset;
                break;
            case DQ.DIR.RIGHT:
                top = -2 * this.yoffset;
                break;
            case DQ.DIR.UP:
                top = -3 * this.yoffset;
                break;
            case DQ.DIR.DOWN:
                top = -0 * this.yoffset;
                break;
            default:
                if (obj.style.backgroundPosition.length != 0) {
                    top = parseInt(obj.style.backgroundPosition.split(" ")[1]);
                }
                break;

        }
        switch (DQ.NPC.pose) {
            case 0:
                left = -0 * DQ.NPC.xoffset;
                break;
            case 1:
                left = -2 * DQ.NPC.xoffset;
                break;
            case 2:
                left = -0 * DQ.NPC.xoffset;
                break;
        }

        obj.style.backgroundPosition = left + "px " + top + "px";
    },
    stand: function (obj) {
        obj = this.obj0;
        var pos = obj.style.backgroundPosition;

        obj.style.backgroundPosition = '-32px ' + pos.split(' ')[1];
    },
    autoWark: function (to, callback) {
        DQ.start();
        var me = this;
        function _moved() {
            if (to.length == 0) {
                callback && callback();
                return;
            }
            setTimeout(function () {
                dir = me.dir = to.shift();
                me.stamp();
                me.moveTo(dir, _moved);
            }, 0);
        }

        dir = this.dir = to.shift();
        this.stamp(dir, true);
        this.moveTo(dir, _moved);
        return;
    }
}

DQ.NPC.wark = function (obj, dir, force) {
}
DQ.NPC.stand = function (obj) {
    var pos = obj.style.backgroundPosition;
    obj.style.backgroundPosition = '-32px ' + pos.split(' ')[1];
}
