﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="../ui/dq-ui.js" />
/// <reference path="dq-rpg.js" />
/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0006
*/

DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg.js", "DQ.RPG.InputBox", function () {

    DQ.Guid = function () { }

    DQ.Guid.prototype = {
        chars: "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
        generate: function () {
            var now = new Date();
            var guid = this.getBase(now);

            var sq = now == this.lastTimestampUsed ? this.counterSeq + 1 : 1;
            for (var i = 0; i < 2; i++) {
                sq += this.chars.charAt(Math.floor(Math.random() * this.chars.length));
            }

            this.lastTimestampUsed = now;

            return guid + sq;
        },
        getBase: function (val) {
            var ch = [];
            while (val != 0) {
                var rm = val % this.chars.length;
                ch.push(this.chars.charAt(rm));
                val = Math.floor(val / this.chars.length);
            }
            return ch.join('');
        }
    }

    DQ.RPG.ItemCollection = dqextend(Array, function () {
    }, {
        owner: null,
        addRange: function (values) {
            for (var i = 0; i < values.length; i++) {
                this.push(values[i]);
            }
        },
        push: function (value) {
            if (value.serial == null) {
                var guid = new DQ.Guid();
                value.serial = guid.generate();
            }
            this.__super.push.apply(this, arguments);
            return value;
        },
        find: function (serial) {
            for (var i = 0; i < this.length; i++) {
                if (this[i].serial == serial) {
                    return this[i];
                }
            }
            return null;
        },
        getItem: function (pos) {
            return world.items.catalog[this[pos].uid];
        },
        remove: function (pos) {
            if (typeof pos == "string") {
                for (var i = 0; i < this.length; i++) {
                    if (this[i].serial == pos) {
                        pos = i;
                        break;
                    }
                }
            }
            else if (pos < 0 || this.length <= pos) {
                throw new Error("DQ.RPG.ItemCollection.remove:Indexの範囲外です");
            }
            this.splice(pos, 1);
        }
    });

    DQ.RPG.Equipments = function (options) {
        options = DQ.options(options, {
            helm: { uid: -1, serial: 0 },
            armor: { uid: -1, serial: 0 },
            weapon: { uid: -1, serial: 0 },
            grobe: { uid: -1, serial: 0 },
            boots: { uid: -1, serial: 0 },
            shield: { uid: -1, serial: 0 },
            accesary: { uid: -1, serial: 0 }
        });
        for (var nm in options) {
            if (!options.hasOwnProperty(nm)) {
                return;
            }
            this[nm] = options[nm];
        }
    }
    DQ.RPG.Equipments.prototype = {
        helm: { uid: -1, serial: 0 },
        armor: { uid: -1, serial: 0 },
        weapon: { uid: -1, serial: 0 },
        grobe: { uid: -1, serial: 0 },
        boots: { uid: -1, serial: 0 },
        shield: { uid: -1, serial: 0 },
        accesary: { uid: -1, serial: 0 }
    };

    DQ.RPG.Gender = { Unknown: 0, Male: 1, Female: 2 };
    DQ.RPG.Creature = function (name, options) {
        ///<summary>
        /// 生き物の基本情報を管理します
        ///</summary>
        ///<param name="name" type="string">
        /// 生き物の名前を指定
        ///</param>
        ///<param name="name" type="object">
        /// オプションを指定
        ///</param>

        options = DQ.options(options, {
            uid: 10001,
            G: 0,
            EXP: 0,
            HP: 0,
            MP: 0,
            LV: 1,
            items: [],
            title: _("Ordinary people"),
            occupation: _("Helo"),
            gender: DQ.RPG.Gender.Male,
            npc: false,
            entryBattle: true

        });
        this._create(name, options);
    }

    DQ.RPG.Creature.prototype = {
        _create: function (name, options) {
            this.name = name;
            this.LV = options.LV || 1;
            this._parmanent = new DQ.RPG.State(options);
            this.temporary = new DQ.Chain(this);

            this.G = options.G;
            this.EXP = options.EXP;
            this.title = options.title;
            this.occupation = options.occupation;
            this.gender = options.gender;
            this.entryBattle = options.entryBattle;
            this.items = new DQ.RPG.ItemCollection();
            this.items.addRange(options.items);
            this.magicIDs = options.magics || [];
            this._equipmentIDs = new DQ.RPG.Equipments();
        },
        temporary: null,
        _parmanent: {},
        parmanent: function (value) {
            if (!arguments.length) {
                return this._parmanent;
            }
            if (value instanceof DQ.RPG.State) {
                this._parmanent = value;
            } else {
                this._parmanent = new DQ.RPG.State(value);
            }
        },
        hasState: function (uid) {
            for (var i = 0; i < this.temporary.length; i++) {
                if (this.temporary[i].uid == uid) {
                    return true;
                }
            }
            return false;
        },
        hp: function (value) {
            if (!arguments.length) {
                return this._parmanent.HP;
            } else {
                this._parmanent.HP = value;
            }
        },
        mp: function (value) {
            if (!arguments.length) {
                return this._parmanent.MP;
            } else {
                this._parmanent.MP = value;
            }
        },
        maxHP: function (value) {
            if (!arguments.length) {
                return this._parmanent.maxHP;
            } else {
                this._parmanent.maxHP = value;
            }
        },
        maxMP: function (value) {
            if (!arguments.length) {
                return this._parmanent.maxMP;
            } else {
                this._parmanent.maxMP = value;
            }
        },
        friend: true, //味方の意
        G: 0,
        LV: 1,
        EXP: 0,
        title: "Ordinary people",
        occupation: "Helo",
        gender: DQ.RPG.Gender.Male,
        npc: false,
        entryBattle: true,
        _command: null,
        command: function (value) {
            if (!arguments.length) {
                return this._command;
            } else {
                this._command = value;
            }
        },
        displayName: function () {
            return this.name;
        },
        incrementItem: function (uid, count, serial) {
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i].uid == uid) {
                    this.items[i].count += count;
                    return true;
                }
            }
            this.items.push({ 'uid': uid, 'count': count });

            //今は無限に追加できる。
            return true;
        },
        decrementItem: function (uid, count) {
            if (arguments.length != 2) {
                return new InvalidArgument("DQ.RPG.Creature.decrementItem: Argument too few.");
            }
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i].uid == uid) {
                    this.items[i].count -= count;
                    if (this.items[i].count <= 0) {
                        this.items.remove(i);
                    }
                    break;
                }
            }
        },
        _equipmentIDs: new DQ.RPG.Equipments(),
        equipmentIDs: function (value) {
            if (!arguments.length) {
                return this._equipmentIDs;
            }
            if (value instanceof DQ.RPG.Equipments) {
                this._equipmentIDs = value;
            } else {
                this._equipmentIDs = new DQ.RPG.Equipments(value);
            }

            var eqps = this._equipmentIDs;
            for (var nm in eqps) {
                if (!eqps.hasOwnProperty(nm) || eqps[nm].uid == -1) {
                    continue;
                }
                for (var i = 0; i < this.items.length; i++) {
                    if (this.items[i].uid == eqps[nm].uid) {
                        eqps[nm].serial = this.items[i].serial;
                        var wi = world.items.catalog[eqps[nm].uid];
                        wi.equip(this);
                        this.items[i].equip = true;
                        break;
                    }
                }
            }
        },
        getSTR: function () {
            var str = this._parmanent.STR;
            for (var i = 0; i < this.temporary.length; i++) {
                str += this.temporary[i].STR;
            }
            return str;
        },
        status: function (name, value) {
            ///<summary>
            /// 素のステータス値を取得または設定します。
            ///</summary>
            ///<param name="name" type="string">
            /// 取得したいステータス名を指定
            ///</param>
            ///<param name="value" type="string">
            /// 新たに設定する値を指定(option)
            ///</param>
            if (value == null) {
                return this._parmanent[name];
            }
            this._parmanent[name] = value;
        },
        levelUp: function (status) {
            for (var nm in status) {
                if (!status.hasOwnProperty(nm)) {
                    continue;
                }
                this._parmanent[nm] != null && (this._parmanent[nm] += parseInt(status[nm]));
            }
        },
        getState: function (name) {
            ///<summary>
            /// 補正後のステータス値を取得します。
            ///</summary>
            ///<param name="name" type="string">
            /// 取得したいステータス名を指定
            ///</param>
            var sts = this._parmanent[name];
            for (var i = 0; i < this.temporary.length; i++) {
                if (this.temporary[i].hasOwnProperty(name)) {
                    sts += this.temporary[i][name];
                }
            }
            return sts;
        },
        getAttack: function () {
            var atk = this.getState('STR');
            for (var nm in this._equipmentIDs) {
                if (!this._equipmentIDs.hasOwnProperty(nm) || typeof this._equipmentIDs[nm] == "function") {
                    continue;
                }
                atk += (this._equipmentIDs[nm] == null || this._equipmentIDs[nm].uid == -1) ?
                0 : world.items.catalog[this._equipmentIDs[nm].uid].ATK;
            }

            var isOverdrive = false;
            for (var i = 0; i < this.temporary.length; i++) {
                var t = this.temporary[i];
                isOverdrive |= t.o_attack == null ? false : t.o_attack;
            }

            return isOverdrive ? Math.floor(atk * 1.5) : atk;
        },
        getDefence: function () {
            var def = this.getState('VIT');
            for (var nm in this._equipmentIDs) {
                if (!this._equipmentIDs.hasOwnProperty(nm)) {
                    continue;
                }
                def += (this._equipmentIDs[nm] == null || this._equipmentIDs[nm].uid == -1) ?
               0 : world.items.catalog[this._equipmentIDs[nm].uid].DEF;
            }

            var isOverdrive = false;
            for (var i = 0; i < this.temporary.length; i++) {
                var t = this.temporary[i];
                def += this.temporary[i].DEF;
                isOverdrive |= t.o_defence == null ? false : t.o_defence;
            }
            return isOverdrive ? Math.floor(def * 1.5) : def;
        },
        canAction: function (reason) {
            return this.temporary.length == 0 ? true : this.temporary.and("canAction", reason);
        },
        canChant: function (reason) {
            return this.temporary.length == 0 ? true : this.temporary.and("canChant", reason);
        },
        gainDamage: function (type, damage) {
            for (var i = 0; i < this.temporary.length; i++) {
                var t = this.temporary[i];
                damage = t.gainDamage(type, damage);
            }
            return damage;
        },
        isAvoid: function (to, type) {
            /// <summary>
            /// 相手の攻撃の回避判定
            /// </summary>

            if (world.isAvoid(this, to)) {
                return true;
            }

            var reason = [];
            for (var i = 0; i < this.temporary.length; i++) {
                var t = this.temporary[i];
                if (t.isAvoid(type, reason)) {
                    return true;
                }
            }
            return false;
        },
        isAvoidMagic: function (to) {
            /// <summary>
            /// 相手の攻撃の回避判定
            /// </summary>

            var agi = this.getState('AGI'),
            genRandom = function (base) {
                return false; //省略
            },
            isAvoid = false;
            for (var i = 0; i < this.temporary.length; i++) {
                var t = this.temporary[i];
                isAvoid |= t.avoid == null ? false : t.avoid;
            }
            return genRandom(isAvoid ? agi + 0.3 : agi);
        },
        isCriticalHit: function (to) {

            //拡張性を考慮してAPIへ委譲
            return world.isCriticalHit(this, to);
        },
        isHit: function (to) {
            /// <summary>
            /// 攻撃のあたり判定
            /// </summary>

            //拡張性を考慮してAPIへ委譲
            return world.isHit(this, to);
        },
        items: new DQ.RPG.ItemCollection(),
        isDead: function () {
            return this._parmanent.HP <= 0;
        },
        magicIDs: [],
        setMagics: function (magics) {
            this.magicIDs = magics.clone();
        },
        setEquipments: function (equipments) {
            ///<summary>
            ///強制的に設定
            ///</summary>
            var eqps = this._equipmentIDs = new DQ.RPG.Equipments();
            eqps.helm = { uid: equipments[0], serial: 0 }
            eqps.armor = { uid: equipments[1], serial: 0 }
            eqps.weapon = { uid: equipments[2], serial: 0 }
            eqps.grobe = { uid: equipments[3], serial: 0 }
            eqps.boots = { uid: equipments[4], serial: 0 }
            eqps.shield = { uid: equipments[5], serial: 0 }
            eqps.accesary = { uid: equipments[6], serial: 0 }
            for (var nm in eqps) {
                if (!eqps.hasOwnProperty(nm) || eqps[nm].uid == -1) {
                    continue;
                }
                var wi = world.items.catalog[eqps[nm].uid];
                wi && wi.equip(this);
                for (var i = 0; i < this.items.length; i++) {
                    if (this.items[i].uid == eqps[nm].uid) {
                        eqps[nm].serial = this.items[i].serial;
                        this.items[i].equip = true;
                        break;
                    }
                }
            }
        },
        dequip: function (type) {
            if (this._equipmentIDs[type] == null) {
                return;
            }
            var item = this.items.find(this._equipmentIDs[type].serial);
            if (item != null) {
                var wi = world.items.catalog[this._equipmentIDs[type].uid];
                wi.remove(this);
                item.equip = false;
            }
            this._equipmentIDs[type].uid = -1;
        },
        equip: function (type, uid, serial) {
            if (this._equipmentIDs[type] == undefined) {
                throw new Error(name + "は、equipmentIDsに含まれません。");
            }

            //はずす
            this.dequip(type);

            //つける
            this._equipmentIDs[type] = { 'uid': uid, 'serial': serial }
            var item = this.items.find(this._equipmentIDs[type].serial);
            if (item != null) {
                var wi = world.items.catalog[uid];
                wi.equip(this);
                item.equip = true;
            }
        },
        escape: function (from) {
            return world.escape(this, from);
        },
        defence: function () {
            for (var i = 0; i < this.temporary.length; i++) {
                if (this.temporary[i].defence) {
                    //重複を回避（でもエラーじゃない）
                    return;
                }
            }
            this.temporary.push(new DQ.RPG.State({ defence: true }));
            return true;
        },
        recoverAll: function () {
            this.hp(this.maxHP());
            this.mp(this.maxMP());
            this.temporary = new DQ.Chain(this);
        },
        update: function (reason) {
            /// <summary>
            /// 状態を更新します。
            /// </summary>

            //状態を更新
            var res = this.temporary.or("update", reason);

            //期限切れの状態は取り除く
            for (var i = this.temporary.length - 1; i >= 0; i--) {
                this.temporary[i].isExpire() && this.temporary.remove(i);
            }
            return res;
        }
    }

    DQ.RPG.dummyState = function (options) {
        options = options || {};

        options.LV = options.LV || 1;
        this.HP = 0;
        this.MP = 0;

        //強さ(Strength)
        options.STR = options.STR || Math.floor(Math.random() * 10 + 1);
        //体力(Vitarity)
        options.VIT = options.VIT || Math.floor(Math.random() * 10 + 1);
        //器用さ(Dexterity)
        options.DEX = options.DEX || Math.floor(Math.random() * 10 + 1);
        //素早さ(Agility)
        options.AGI = options.AGI || Math.floor(Math.random() * 10 + 1);
        //賢さ(Wisdom)
        options.WIS = options.WIS || Math.floor(Math.random() * 10 + 1);
        //賢さ(Intelligence)
        options.INT = options.INT || Math.floor(Math.random() * 10 + 1);
        //魅力(Charm)
        options.CHA = options.CHA || Math.floor(Math.random() * 10 + 1);

        options.HP = options.HP || this.HP + options.VIT + options.LV * 4;
        options.maxHP = options.HP;

        options.MP = options.MP != null ? options.MP : this.MP + options.WIS + options.LV * 4;
        options.maxMP = options.MP;

        return options;
    }
    DQ.RPG.State = function (options) {
        options = DQ.options(options, {
            LV: 1,
            uid: -1,
            HP: 0,
            MP: 0,
            STR: 0,
            VIT: 0,
            DEX: 0,
            AGI: 0,
            WIS: 0,
            INT: 0,
            CHA: 0,
            ATK: 0,
            DEF: 0,
            maxHP: options.HP,
            maxMP: options.MP,
            name: "",
            a_text: "",
            e_text: "",
            text: "",
            description: "",
            attack: true,
            magic: true,
            count: 0,
            good: false
        });
        for (var nm in this) {
            if (!options.hasOwnProperty(nm)) {
                continue;
            }
            if (options[nm] instanceof Array) {
                this[nm] = options[nm];
            } else {
                if (options[nm] == "" || typeof options[nm] == "boolean" || isNaN(options[nm])) {
                    this[nm] = options[nm];
                } else {
                    this[nm] = parseInt(options[nm]);
                }
            }
        }

        this._strategy = this._buildStrategy(options);
    }
    DQ.RPG.State.prototype = {
        owner: null,
        uid: -1,
        ATK: 0,
        DEF: 0,
        HP: 0,
        MP: 0,
        maxHP: 0,
        maxMP: 0,
        STR: 1,
        VIT: 1,
        DEX: 1,
        AGI: 1,
        WIS: 1,
        INT: 1,
        CHA: 1,
        attack: true,
        magic: true,
        poison: false,
        stan: false,
        defence: false,
        stone: false,
        holy: false,
        slow: false,
        sleep: false,
        silent: false,
        dark: false,
        count: 0,
        good: false,
        name: "",
        text: "",
        a_text: "",
        e_text: "",
        _buildStrategy: function (state) {
            if (state.poison) {
                return new DQ.RPG.State.Poison(this);
            } else if (state.stan) {
                return new DQ.RPG.State.Stan(this);
            } else if (state.defence) {
                return new DQ.RPG.State.Defence(this);
            } else {
                for (var i = 0; i < DQ.RPG.State.table.length; i++) {
                    if (DQ.RPG.State.table[i].uid == state.uid) {
                        return new DQ.RPG.State.table[i].strategy(this);
                    }
                }
                return new DQ.RPG.State.StateStg(this);
            }
        },
        _strategy: {},
        isExpire: function () {
            return this._strategy.isExpire();
        },
        isAvoid: function (type, reason) {
            return this._strategy.isAvoid(type, reason);
        },
        gainDamage: function (type, damage) {
            return this._strategy.gainDamage(type, damage);
        },
        canAction: function (reason) {
            //行動可否
            return this._strategy.canAction(reason);
        },
        canChant: function (reason) {
            //行動可否
            return this._strategy.canChant(reason);
        },
        enter: function (reason) {
            return this._strategy.enter(this.owner, reason);
        },
        leave: function (reason) {
            return this._strategy.leave(this.owner, reason);
        },
        update: function (reason) {
            return this._strategy.update(this.owner, reason);
        }
    }

    DQ.RPG.State.table = [];

    DQ.RPG.State.StateStg = function (owner) {
        this.owner = owner;
    }
    DQ.RPG.State.StateStg.prototype = {
        _expire: false,
        isExpire: function () {
            return this._expire;
        },
        canAction: function (reason) {
            if (!this._expire && !this.owner.attack && !this.owner.magic) {
                //reason && reason.push(this.onwer.text);
                return false;
            }
            return true;
        },
        canChant: function (reason) {
            if (!this._expire && !this.owner.magic) {
                //reason && reason.push(this.onwer.text);
                return false;
            }
            return true;
        },
        isAvoid: function (type, reason) {
            return false;
        },
        gainDamage: function (type, damage) {
            return damage;
        },
        enter: function (owner, reason) {
        },
        leave: function (owner, reason) {
        },
        update: function (owner, reason) {
            this._expire = world.isExpireStatus(owner, this.owner);
            if (this._expire) {
                if (this.owner.uid == 10006) {
                    world.party.flag[DQ.RPG.Party.FlagPos.Holy] = false;
                }
                reason.push(DQ.format(this.owner.e_text, owner.name));
            }
            return true;
        }
    }
    DQ.RPG.State.Poison = function (owner) {
        this.owner = owner;
    }
    DQ.RPG.State.Poison.prototype = {
        isExpire: function () {
            return false;
        },
        canAction: function () {
            return true;
        },
        canChant: function (reason) {
            if (!this._expire && !this.owner.magic) {
                //reason && reason.push(this.onwer.text);
                return false;
            }
            return true;
        },
        enter: function (owner, reason) {
            this._count = 1;
            this.update(owner, reason);
        },
        leave: function () {
            return true;
        },
        isAvoid: function (type, reason) {
            return false;
        },
        gainDamage: function (type, damage) {
            return damage;
        },
        update: function (owner, reason) {
            var to = owner;
            this._count--;
            if (this._count) {
                return false;
            }
            var hp = to.hp() - this.owner.HP >= 0 ? this.owner.HP : to.hp();
            to.hp(to.hp() - hp);
            reason.push(to.name +
                    "は毒に侵されている。\n" + hp + "ポイントのダメージを受けた。");

            this._count = this.owner.count;
            return true;
        }
    }

    DQ.RPG.State.Defence = function (owner) {
        this.owner = owner;
    }
    DQ.RPG.State.Defence.prototype = {
        isExpire: function () {
            return this._count <= 0;
        },
        canAction: function () {
            return true;
        },
        isAvoid: function (type, reason) {
            return false;
        },
        gainDamage: function (dmg, type) {
            if (type == DQ.RPG.DamageProperty.Pysical && dmg > 1) {
                return Math.floor(dmg / 2);
            }

            return dmg;
        },
        enter: function (owner, reason) {
            this._count = 1;
            reason.push(DQ.format(this.owner.a_text, owner.name));
        },
        leave: function () {
            return true;
        },
        update: function (owner, reason) {
            this._count--;
            return true;
        }
    }

    DQ.RPG.MonsterCatalog = function (options) {
        /// <summary>
        ///     モンスターのカタログを管理
        /// </summary>

        options = DQ.options(options, {
            url: "data/monster.txt",
            plane: false,
            callback: null
        });

        this._plane = options.plane;
        this.onLoaded = options.callback;
        this.url = options.url;
        this.load(this.url);
    }

    DQ.RPG.MonsterCatalog.prototype = {
        _plane: false,
        catalog: [],
        load: function (url) {
            var me = this;
            DQ.loadCatalog(url, function (data) {
                try {
                    eval('var s=' + data);
                } catch (e) {
                    alert("MonsterCatalog::load() " + e);
                    me.onLoaded && me.onLoaded(me, false);
                }
                me.version = s.version;
                me._data = s.catalog || s.data;
                me.catalog.length = 0;
                me.catalog = [];
                var _genUID = function () {
                    var uid = 10001;
                    var catalog = me.catalog;
                    for (var i = 10001; i < catalog.length; i++) {
                        if (!catalog[i]) {
                            uid = i;
                            break;
                        }
                        if (uid < catalog[i].uid) {
                            uid = catalog[i].uid + 1;
                        }
                    }
                    return uid;
                }
                for (var i = 0; i < me._data.length; i++) {
                    if (me._data[i].uid == null) {
                        me._data[i].uid = _genUID();
                    }
                    if (!me._data[i].items || me._data[i].items.length == 0) {
                        me._data[i].items = [-1, -1, -1, -1, -1];
                    }
                    if (!me._data[i].magics || me._data[i].magics.length == 0) {
                        me._data[i].magics = [-1, -1];
                    }
                    if (me._plane) {
                        me.catalog[me._data[i].uid] = me._data[i];
                    } else {
                        me.catalog[me._data[i].uid] = new DQ.RPG.Monster(me._data[i].name, me._data[i]);
                    }
                }
                me.onLoaded && me.onLoaded.apply(me);
            });
        },
        create: function (value, suffix) {
            var mo = null;
            if (typeof value == "number") {
                mo = this._data[value];
            } else {
                for (var i = 0; i < this._data.length; i++) {
                    if (this._data[i].text == value || this._data[i].name == value) {
                        mo = this._data[i];
                        break;
                    }
                }
            }
            if (!mo) {
                throw "指定のmonsterはいない:" + value;
            }
            if (suffix == null) {
                suffix = "";
            }
            var mon = new DQ.RPG.Monster(mo.text + suffix, mo);
            mon.hp(mon.hp() + DQ.genRandom(mon.hp()));
            return mon;
        },
        find: function (name, value) {
            ///<summary>
            /// モンスターを検索します。
            ///</summary>
            ///<param name="name" type="string">
            ///検索に使用するプロパティーを指定(option)
            ///</param>
            ///<param name="value" type="string/number">
            ///検索する値を指定
            ///</param>

            if (arguments.length == 1) {
                value = name;
                name = "title";
            }
            var res = null;
            for (var i = 10001; i < this.catalog.length; i++) {
                if (this.catalog[i] && this.catalog[i][name] == value) {
                    return this.catalog[i];
                }
            }
        },
        findIndex: function (title) {
            var res = null;
            var idx = 0;
            for (var i = 10001; i < this.catalog.length; i++, idx++) {
                if (this.catalog[i] && this.catalog[i].title == title) {
                    return idx;
                }
            }
            return -1;
        },
        choose: function (titles) {
            var r = Math.floor(Math.random() * titles.length);
            return this.find(titles[r]);
        }
    }

});