﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="dq-rpg.js" />
/// <reference path="dq-battle.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: (http://www.opensource.org/licenses/mit-license.php)
*
* Date: 2011-07-01 17:34:21 -0900
* Revision: 0003
*/

DQ.lazyLoad(DQ.subdir + "rpg/dq-battle.js", "DQ.RPG.BEngine");
DQ.lazyLoad(DQ.subdir + "dq-serial.js", "DQ.JsonSerializer");
DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg.js", "DQ.RPG.InputBox", function () {
    DQ.SCALE = 1;
    DQ.QUORITY = 2;
    DQ.DOM = false;

    DQ._message['ja']["New GAME"] = "はじめから";
    DQ._message['ja']["CONTINUE"] = "つづきから";
    DQ._message['ja']["GAME START"] = "ゲーム開始";
    DQ._message['ja']["Mr.Unknown"] = "ななし";
    DQ._message['ja']["Cancel"] = "やめる";
    DQ._message['ja']["Auto save"] = "自動保存データ";

    DQ.RPG.Engine = function (options) {
        /// <summary>
        /// DQII風エンジン
        /// </summary>
        options = DQ.options(options, {
            projectName: "default",
            small: false,
            titleImage: "title",
            gameoverImage: "gameover"
        });
        this.projectName = options.projectName;
        this.small = options.small;
        this.titleImage = options.titleImage;
        this.gameoverImage = options.gameoverImage;
    }
    DQ.RPG.Engine.prototype = {
        dir: 0,
        kp: null,
        mp: null,
        small: false,
        project: null,
        projectName: "default",
        titleImage: "title",
        gameoverImage: "gameover",
        setupSoundBox: function () {
            var set = [],
                catalog = world.sounds.catalog.data || world.sounds.catalog.catalog;
            for (var i = 0; i < catalog.length; i++) {
                var s = catalog[i];
                if (s.jingle) {
                    s.src.name = s.name;
                    set.push(s.src);
                }
            }
            world.soundBox().addRange(set);
        },
        initialize: function (name, callback) {
            var me = this;
            world = new DQ.World();
            world.debug = DQ.queryString["debug"] == "true" ? true : false;
            world.initialized = false;
            world.version = "0.6.0.0825";
            world.engine = this;
            world.onSave = this.save;
            world.onLoad = this.load;
            world.onGameOver = function () { me.gameOver.apply(me); }

            var count = 0;
            function _loaded() {
                count++;
                if (count == 2) {
                    me.setupSoundBox();
                    callback();
                }
            }
            world.project = this.project = new DQ.RPG.Project({
                name: this.projectName,
                callback: _loaded
            });


            page = DQ.page();

            //スクリーンを作成
            var v = $('#' + name);

            //マップの座標表示 - for DEBUG
            v.append($('#mater'));

            var style = v[0].currentStyle || document.defaultView.getComputedStyle(v[0], '')
            var sc = world.screen = new DQ.Screen(DQ.page(), parseInt(style.width), parseInt(style.height), name);

            //タイトル表示演出
            sc.showTitle(world.project.name + "/images/" + this.titleImage + ".png", _loaded);

            //マップ
            if (!DQ.DOM) {
                this.mp = world.map = new DQ.Screen.MapByCanvas(sc, {
                    delta: 4 / DQ.QUORITY
                });
            } else {
                this.mp = world.map = new DQ.Screen.Map(sc, {
                    delta: 4 / DQ.QUORITY
                });
            }

            //キーパッドを作成
            this.kp = new DQ.KeyPad(sc, 288 / DQ.SCALE, 16, {
                left: _("Left"),
                right: _("Right"),
                up: _("Up"),
                down: _("Down")
            });

            //ショップ
            world.shop = new DQ.RPG.Shop(sc)

            //ホテル
            world.hotel = new DQ.RPG.Hotel(sc)

            //プレイヤーのパーティ
            world.party = new DQ.RPG.Party();


            //画面表示インスタンスの作成
            this.visual = new DQ.RPG.Visual();
            this.visual.setup(this.small);
            world.levelBox = this.visual.lvBox;

            //プロジェクトロード
            this.project.load();

        },
        createNewPlayer: function (name, uid) {
            var player;
            if (world.debug) {
                player = new DQ.RPG.Player(name, world.levels.find('uid', uid).levels[29]);
            } else {
                player = new DQ.RPG.Player(name, world.levels.find('uid', uid).levels[0]);
            }
            player.uid = uid;
            world.party.push(player);

            //デバッグ用に初期保有アイテムを設定
            if (world.debug) {
                world.party.push(dq_temp_player());

                world.party[0].items.addRange([
                    { uid: 10001, count: 1 },
                    { uid: 10002, count: 2 },
                    { uid: 10003, count: 3 },
                    { uid: 10004, count: 4 },
                    { uid: 10005, count: 5 },
                    { uid: 10006, count: 6 },
                    { uid: 10007, count: 7 },
                    { uid: 20001, count: 1 },
                    { uid: 20002, count: 1 },
                    { uid: 30001, count: 1 },
                    { uid: 30002, count: 1 },
                    { uid: 40001, count: 1 },
                    { uid: 40002, count: 1 }
                ]);
                world.party[0].setMagics([10001, 10002, 10003, 10004, 10005, 10007]);
                world.party[0].setEquipments([-1, 30001, 20001, -1, -1, 40001, -1]);
            }
            this.visual.lvBox.party(world.party);
        },
        gameOver: function () {
            var me = this;

            //背景
            world.screen.showBackgroundImage(world.project.name + "/images/" + this.gameoverImage + ".png");

            //サウンド
            var sound = world.sounds.find("name", "over");
            sound && world.playSound(sound.src);

            //メッセージ
            var _msg = world.messages.find('uid', 30001);
            world.screen.showMessage(_msg.message[0], function () {
                setTimeout(function () {
                    world.screen.hideTitle();
                    world.engine.startOpening(true);
                }, 0);
            });
        },
        load: function (sender, pos) {
            ///<summary>
            /// データーをロードする
            ///</summary>
            var _load = function () {
                var me = world.engine;
                try {
                    var data = {}
                    switch (pos) {
                        case 0:
                            data = {
                                G0: 0,
                                encount0: world.steps,
                                flag0: [],
                                party_flag0: [],
                                map0: world.events.map,
                                X0: me.mp.mapX,
                                Y0: me.mp.mapY,
                                sound0: {},
                                party0: []
                            };
                            break;
                        case 1:
                            data = {
                                G1: 0,
                                encount1: world.steps,
                                flag1: [],
                                party_flag1: [],
                                map1: world.events.map,
                                X1: me.mp.mapX,
                                Y1: me.mp.mapY,
                                sound1: {},
                                party1: []
                            };
                            break;
                        case 2:
                            data = {
                                G2: 0,
                                encoun2t: world.steps,
                                flag2: [],
                                party_flag2: [],
                                map2: world.events.map,
                                X2: me.mp.mapX,
                                Y2: me.mp.mapY,
                                sound2: {},
                                party2: []
                            };
                            break;
                        default:
                            data = {
                                G: 0,
                                encount: world.steps,
                                flag: [],
                                party_flag: [],
                                map: world.events.map,
                                X: me.mp.mapX,
                                Y: me.mp.mapY,
                                sound: {},
                                party: []
                            };
                            break;
                    }
                    if (pos == null) {
                        pos = "";
                    }
                    world.localData(data);
                    DQ.load(world);
                    if (!world.data['flag' + pos]) {
                        return false;
                    }

                    world.party = new DQ.RPG.Party();
                    world.steps = parseInt(world.data['encount' + pos]);
                    world.flag = eval("[" + world.data['flag' + pos] + "]");
                    world.party.flag = eval("[" + world.data['party_flag' + pos] + "]");
                    world.party.G = world.data['G' + pos] != null ? parseInt(world.data['G' + pos]) : 0;

                    var sel = new DQ.JsonSerializer(DQ.RPG.Player);
                    if (world.data['party' + pos]) {
                        var p = world.data['party' + pos];
                        world.data['party' + pos] = "[" + p + "]";
                        var pt = world.data['party' + pos];
                        eval("rr = " + pt);
                        for (var i = 0; i < rr.length; i++) {
                            var p = sel.deserialize(rr[i]);
                            p.friend = true;
                            world.party.push(p);
                        }
                        world.party[0].entryBattle = true;
                        world.debug &&
                    world.party.length == 1 && world.party.push(dq_temp_player());
                    }
                    me.visual.lvBox.party(world.party);


                    //world.party[0].incrementItem(10005, 1);
                    if (world.debug) {
                        world.party[0].incrementItem(10010, 1);
                        world.party[0].setMagics([10001, 10002, 10003, 10004, 10005, 10006, 10007]);
                        world.party[0]._parmanent.maxMP = world.party[0]._parmanent.MP = 100;
                    }
                    return true;
                }
                catch (e) {
                    return false;
                }
            }

            var res = _load();
            if (!res) {
                var back = world.project.name;
                world.project.name = "";
                res = _load();
                world.project.name = back;
            }

            return res;
        },
        newGame: function () {
            //新規
            world.resetFlags();
            world.party = new DQ.RPG.Party();

            //名前入力
            var me = this;
            var ib = new DQ.RPG.InputBox(world.startPage, { top: 192 / DQ.SCALE, left: DQ.LV2, width: '13em' });
            ib.onClick = function _inputBox_click(sender) {
                var name = sender.value() == "" ? _("Mr.Unknown") : sender.value();
                me.createNewPlayer(name, 10001);

                //初期のマップを設定
                world.events.setup(10000);
                world.events.playSound();
                world.maps.find("name", world.events.map).load();
            }
        },
        save: function (sender, pos) {
            ///<summary>
            /// データーをセーブする
            ///</summary>
            var me = this;
            if (!world || !world.initialized || world.party == null) {
                //中途半端な場内なら保存しない
                return;
            }

            var p = world.party[0];
            var sel = new DQ.JsonSerializer(DQ.RPG.Player);
            var pt = [];
            for (var i = 0; i < world.party.length; i++) {
                try {
                    pt.push(sel.serialize(world.party[i]));
                } catch (e) {
                    return;
                }
            }
            var data;
            switch (pos) {
                case 0:
                    data = {
                        encount0: world.steps,
                        flag0: world.flag,
                        party_flag0: world.party.flag,
                        map0: world.events.map,
                        X0: world.map.mapX,
                        Y0: world.map.mapY,
                        party0: pt,
                        G0: world.party.G
                    };
                    break;
                case 1:
                    data = {
                        encount1: world.steps,
                        flag1: world.flag,
                        party_flag1: world.party.flag,
                        map1: world.events.map,
                        X1: world.map.mapX,
                        Y1: world.map.mapY,
                        party1: pt,
                        G1: world.party.G
                    };
                    break;
                case 2:
                    data = {
                        encount2: world.steps,
                        flag2: world.flag,
                        party_flag2: world.party.flag,
                        map2: world.events.map,
                        X2: world.map.mapX,
                        Y2: world.map.mapY,
                        party2: pt,
                        G2: world.party.G
                    };
                    break;
                default:
                    data = {
                        encount: world.steps,
                        flag: world.flag,
                        party_flag: world.party.flag,
                        map: world.events.map,
                        X: world.map.mapX,
                        Y: world.map.mapY,
                        party: pt,
                        G: world.party.G
                    };
                    break;
            }
            world.localData(data);
            DQ.save(world);
        },
        setupFrame: function () {
            var delta = Math.floor(8 / DQ.QUORITY);
            DQ.fps(16 * DQ.QUORITY);
            DQ.NPC._frame = 4 * DQ.QUORITY;
            world.map.frame = 4 * DQ.QUORITY;
            world.chara.frame = 2 * DQ.QUORITY;
            world.map.deltaX = delta;
            world.map.deltaY = delta;
            for (var i = 0; i < world.map.layer.length; i++) {
                world.map.layer[i].deltaX = delta;
                world.map.layer[i].deltaY = delta;
            }
        },
        startOpening: function (restart) {
            /// <summary>
            /// オープニング画面を表示
            /// </summary>

            //戦闘システムロード
            //(モンスターカタログを必要とするのでここで作成)
            if (!restart) {
                world.battle = new DQ.RPG.BEngine(world.screen, world.monsters);
                DQ.chain.push(this.mp);
                DQ.chain.push(world.battle);
            }

            world.screen.showTitle(world.project.name + "/images/" + this.titleImage + ".png");
            world.startPage.obj
                .append($('<div>')
                .text('version ' + world.version).css({ position: 'relative', top: 160 / DQ.SCALE, left: '160px' }));

            //BGM再生
            var sound = world.sounds.find("name", "opening");
            sound && world.playSound(sound.src);

            this.small && this.mp.small();
            var me = this;
            var lmenu = new DQ.StateBox(world.screen, {
                width: '14em',
                height: 'auto',
                left: DQ.L2,
                top: 240 / DQ.SCALE,
                canSelect: true,
                data: [{ text: _("Slot#1") }, { text: _("Slot#2") }, { text: _("Slot#3") }, { text: _("Auto save")}],
                text: _("CONTINUE"),
                click: function lmenu$click(sender, pos) {
                    lmenu.hide();
                    pmenu.hide();
                    pos = pos == 3 ? "" : pos;
                    if (me.load(sender, pos)) {
                        //ロード成功
                        world.jump(world.data['map' + pos], parseInt(world.data['X' + pos]), parseInt(world.data['Y' + pos]));
                        return;
                    }
                    me.newGame();
                }
            });
            var pmenu = new DQ.StateBox(world.screen, {
                width: '14em',
                height: 'auto',
                left: DQ.L2,
                top: 192 / DQ.SCALE,
                canSelect: true,
                canCancel: false,
                data: [{ text: _("New GAME") }, { text: _("CONTINUE")}],
                text: _("GAME START"),
                click: function pmenu$click(sender, pos) {
                    if (pos == 1) { //CONTINUE
                        //セーブデータの説明を作成
                        var data = [];
                        for (var i = 0; i < 3; i++) {
                            //データを一度ロードして情報を取得
                            if (me.load(sender, i)) {
                                data.push({ 'text': world.party[0].name + " LV" + world.party[0].LV, enabled: true });
                            } else {
                                data.push({ 'text': _("Slot#" + (i + 1)), enabled: false });
                            }
                        }
                        data.push({ 'text': _("Auto save"), enabled: true });
                        lmenu.setData(data);
                        lmenu.show();
                        lmenu.bringToFront(9999);
                        return;
                    }
                    pmenu.hide();
                    me.newGame();
                }
            });

            function _loaded() {
            }

            //初期メニュー表示
            pmenu.show(true);
            //黒レイヤーよりも上に移動
            pmenu.bringToFront(9999);

            me.onOpening && me.onOpening();
        },
        startGame: function () {
            ///<summary>
            /// オープニング画面を閉じて、開始する。
            ///</summary>
            var me = this;
            world.chara.height(world.project.charaHeight);
            world.map.npcHeight(world.project.npcHeight);

            world.screen.msgBox().playerName = world.party[0].name;

            //開始ページを削除
            world.screen.hideTitle();
            world.initialized = true;

            //平常時のPlayerの状態表示
            this.visual.showLvBox();

            //インターバルタイマーにマップを追加
            DQ.start();

            //本編用キーイベントを設定
            DQ.document.keyup(function (e) {
                me.keydown = false;
                me.dir = DQ.DIR.NONE;
                me.mp.stop();
                return false;
            });

            DQ.document.keydown(function (e) {
                //バブリングしないようにしているはずなので、メニュー表示中は
                //このイベントハンドラはfireされないはず。

                if (me.keydown) {
                    return true;
                }
                me.keydown = true;

                //メニュー表示中は無視
                if (me.visual.commandBox.visible() ||
                    world.battle.battle || world.shop.opend || world.hotel.opend) {
                    return true;
                }

                if (e.altKey || e.ctrlKey || e.shiftKey) {
                    return true;
                }
                e.stopImmediatePropagation();

                $('#ky').text(e.which); //for DEBUG

                //半チップ移動中は抑止
                if (!me.mp.scrollig && e.which == 86) { // 'v'key
                    me.dir = DQ.DIR.NONE;
                    var sound = world.sounds.find('name', "menu");
                    sound && world.playJingle2(sound.src, false);
                    me.visual.commandBox.show();
                    return false;
                }

                var temp = 0;
                switch (e.which) {
                    case 38:
                        temp = DQ.DIR.UP;
                        break;
                    case 40:
                        temp = DQ.DIR.DOWN;
                        break;
                    case 37:
                        temp = DQ.DIR.LEFT;
                        break;
                    case 39:
                        temp = DQ.DIR.RIGHT;
                        break;
                }
                world.chara.wark(temp, true);


                if (temp && world.timerID) {
                    clearTimeout(world.timerID);
                    delete world.timerID;
                }


                if (temp != 0 && temp != me.dir) {
                    me.dir = temp;
                    me.mp.scroll(me.dir);
                    return false;
                }

                return true;
            });
        },
        start: function (formName) {
            ///<summary>
            ///開始します
            ///</summary>
            var me = this;

            this.projectName = DQ.queryString["project"] || "default";

            //画面初期化
            this.initialize(formName, function () { me.startOpening.apply(me); });
            $('.dq-form-content').css({ 'overflow': 'hidden' });

            //イベントハンドラ設定
            this.keydown = false;
            this.kp.mousedown(function (dir, e) {
                /// <summary>
                /// キーパッド押下操作
                /// </summary>

                me.dir = dir;
                me.keydown = true;
                world.chara.wark(dir, true);
                me.mp.scroll(dir);
                $('#sx').text(me.mp.mapX);
                $('#sy').text(me.mp.mapY);
                e.stopImmediatePropagation();
                return false;
            });

            this.kp.mouseup(function (dir, e) {
                /// <summary>
                /// キーパッド開放操作
                /// </summary>
                e.stopImmediatePropagation();
                me.keydown = false;
                me.mp.scroll(0);
                return false;
            });

            this.mp.onLoaded = function (mp) {
                /// <summary>
                ///  マップ読み込み完了イベントハンドラ
                /// </summary>

                //for DEBUG
                $('#sx').text(me.mp.mapX + DQ.OX);
                $('#sy').text(me.mp.mapY + DQ.OY);

                //レイヤー１のイベントを設定＆描画
                var hi = me.mp.layer[0];
                for (var nm in world.events.events) {
                    hi.data[nm] = world.events.events[nm];
                }
                hi.forceMap();

                //視野を設定する
                if (!world.map.dungeon) {
                    world.sunshine();
                } else {
                    if (world.party.flag[DQ.RPG.Party.FlagPos.Torch]) {
                        !world._light && world.lightUp();
                    } else {
                        world.lightDown();
                    }
                }

                world.steps = 255;

                //オープニング画面からのロードなら開始
                world.startPage && me.startGame();

                //マップ進入イベント
                world.events.enter && world.events.enter.fire(world);

                //ゲーム開始イベント
                world.events.start && world.events.start.fire(world);
            }

            this.mp.onScrolled = function (mp, dir) {
                /// <summary>
                /// スクロール完了イベントハンドラ
                /// </summary>

                //for DEBUG
                $('#sx').text(me.mp.mapX + DQ.OX);
                $('#sy').text(me.mp.mapY + DQ.OY);

                if (!me.keydown && !world.timerID) {
                    //猶予期間後スライドダウン
                    world.timerID = setTimeout(function (e) {
                        if (!me.mp.scrolling && !me.keydown) {
                            me.visual.slideDownLvBox();
                            world.chara.stand();
                            delete world.timerID;
                        } else {
                            world.timerID = setTimeout(arguments.callee, DQ.LV_LAZY);
                        }
                    }, DQ.LV_LAZY);
                }

                function _eventEnd() {
                }

                //イベント判定
                if (me.mp.location.target.length == 0) {
                    //半チップずれが発生していない
                    var xx = me.mp.mapX + DQ.OX,
                        yy = me.mp.mapY + DQ.OY;

                    if (world.events.fire([xx, yy], _eventEnd)) {
                        stop_scroll();
                        return false;
                    }
                }

                //歩数に伴うパーティの状態を更新
                var reason = [];
                world.party.update(reason);
                if (reason.length) {
                    world.screen.showMessage(reason);

                    //移動停止
                    stop_scroll();
                    return false; //エンカウント判定はスキップ
                }

                //遭遇判定
                var ne = world.isEncount(me.mp.mapX + DQ.OX, me.mp.mapY + DQ.OY);
                $('#en').text(world.steps);
                if (ne && me.mp.encount) {

                    //移動停止
                    stop_scroll();

                    //準備
                    var mons = world.chooseMonster(me.mp.mapX + DQ.OX, me.mp.mapY + DQ.OY);
                    if (world.canEncount(mons)) { //聖水効果判定
                        world.startBattle({
                            sound: "battle2",
                            enemy: mons,
                            lvBox: me.visual.lvBox,
                            callback: world.postBattle
                        });
                    } else {
                        debug.writeln("escape by holy water.");
                    }
                    return false;
                } else {
                    if (me.keydown) {
                    } else {
                        return false;
                    }
                }

                return true;
            }

            function stop_scroll() {
                me.mp.stop();
                me.dir = 0;
                me.mp.scroll(null);
                me.visual.slideDownLvBox();
                if (world.timerID) {
                    clearTimeout(world.timerID);
                    delete world.timerID;
                }
            }

            function delaySlideDown() {
                if (!world.timerID) {
                    world.timerID = setTimeout(function (e) {
                        if (!me.mp.scrolling && !me.keydown) {
                            me.visual.slideDownLvBox();
                            world.chara.stand();
                            delete world.timerID;
                        } else {
                            world.timerID = setTimeout(arguments.callee, DQ.LV_LAZY);
                        }
                    }, DQ.LV_LAZY);
                }
            }

            this.mp.onPrescroll = function (mp, dir, delta) {
                /// <summary>
                ///     スクロール前イベントハンドラ
                /// </summary>

                var x = me.mp.mapX + DQ.OX,
                    y = me.mp.mapY + DQ.OY,
                    cancel = false;

                function _eventEnd() {
                }

                //イベントチェックと障害物判定
                var target = me.mp.location.target;
                for (var i = 0; i < target.length; i++) {
                    var xx = x + target[i][0],
                        yy = y + target[i][1];

                    if (world.events.isPreEvent([xx, yy])) {
                        //進む前に処理すべきイベント
                        if (world.events.fire([xx, yy], _eventEnd)) {
                            //進めない
                            delaySlideDown();
                            return true;
                        }
                    }

                    if (me.mp.data[yy] == undefined) { //エラー回避
                        delaySlideDown();
                        return true;
                    }

                    var p = me.mp.data[yy][xx];
                    var h = me.mp.layer1[yy][xx];
                    cancel = p == undefined || me.mp.canMove[p] == undefined || !me.mp.canMove[p] || me.mp.canMoveT[h] == 0 || cancel;
                }

                if (!cancel) {
                    me.visual.slideUpLvBox();
                } else {
                    delaySlideDown();
                }

                $('#st').text(me.mp.data[y][x]);
                return cancel;
            }

            me.mp.onScrolling = function (sender, dir) {
                //プレイヤー絵移動
                world.chara.wark(dir);
            }
            me.mp.onBorder = function (sender, dir) {
                //マップの外壁
                world.events.exit && world.events.exit.fire(world);
            }
        }

    }

    //ページアンロードで保存
    $(window).unload(function () {
        world && world.engine.save();
    });
});