﻿/// <reference path="jquery.js" />
/// <reference path="dq.js" />
/// <reference path="dq-rpg.js" />

/*
** DQ Retro game UI framework JavaScript library version v0.6.0
**
** Copyright (c) 2009 M., Koji
** Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
**
** Date: 2009-11-01 17:34:21 -0900
** Revision: 0004
**
**/

DQ.lazyLoad(DQ.subdir + "rpg/dq-rpg-creature.js", "DQ.RPG.MonsterCatalog", function () {
    DQ.RPG.Player = dqextend(DQ.RPG.Creature, function (name, options) {
        /// <summary>
        /// 人（基本的にプレイヤー）を定義およびステータスを管理します。
        /// </summary>
        /// <param name="name" type="string">
        /// 名前を指定
        /// </param>
        this.base(name, options);
    }, {
        uid: 10001,
        friend: true,
        getEquipmentString: function () {
            /// <summary>
            ///     装備品の一覧を取得
            /// </summary>
            /// <return type="string[]">
            var eqps = this._equipmentIDs;
            var eq = [];
            var catalog = world.items.catalog;

            eq.push(!eqps.helm || eqps.helm.uid == -1 ? "そうびなし" : catalog[eqps.helm.uid].name);
            eq.push(!eqps.armor || eqps.armor.uid == -1 ? "そうびなし" : catalog[eqps.armor.uid].name);
            eq.push(!eqps.weapon || eqps.weapon.uid == -1 ? "そうびなし" : catalog[eqps.weapon.uid].name);
            eq.push(!eqps.grobe || eqps.grobe.uid == -1 ? "そうびなし" : catalog[eqps.grobe.uid].name);
            eq.push(!eqps.boots || eqps.boots.uid == -1 ? "そうびなし" : catalog[eqps.boots.uid].name);
            eq.push(!eqps.shield || eqps.shield.uid == -1 ? "そうびなし" : catalog[eqps.shield.uid].name);
            eq.push(!eqps.accesary || eqps.accesary.uid == -1 ? "そうびなし" : catalog[eqps.accesary.uid].name);

            return eq;
        },
        getWeaponList: function () {
            /// <summary>
            ///     武器の一覧(ID)を取得する
            /// </summary>
            /// <return type="number[]">
            var wp = [];
            var catalog = world.items.catalog;
            var items = this.items;

            for (var i = 0; i < items.length; i++) {
                if (catalog[items[i].id].data.category == DQ.RPG.Item.Category.Weapon) {
                    wp.push(items[i].id);
                }
            }

            return wp;
        },
        getArmorList: function () {
            /// <summary>
            ///     防具の一覧(ID)を取得する
            /// </summary>
            /// <return type="number[]">
            var wp = [];
            var catalog = world.items.catalog;
            var items = this.items;

            for (var i = 0; i < items.length; i++) {
                if (catalog[items[i].id].data.category == DQ.RPG.Item.Category.Armor) {
                    wp.push(items[i].id);
                }
            }

            return wp;
        },
        getShieldList: function () {
            /// <summary>
            ///     盾の一覧(ID)を取得する
            /// </summary>
            /// <return type="number[]">
            var wp = [];
            var catalog = world.items.catalog;
            var items = this.items;

            for (var i = 0; i < items.length; i++) {
                if (catalog[items[i].id].data.category == DQ.RPG.Item.Category.Shield) {
                    wp.push(items[i].id);
                }
            }

            return wp;
        },
        getAccesaryList: function () {
            /// <summary>
            ///     盾の一覧(ID)を取得する
            /// </summary>
            /// <return type="number[]">
            var wp = [];
            var catalog = world.items.catalog;
            var items = this.items;

            for (var i = 0; i < items.length; i++) {
                if (catalog[items[i].id].data.category == DQ.RPG.Item.Category.Accesary) {
                    wp.push(items[i].id);
                }
            }

            return wp;
        },
        dropItem: function (id, count) {
            /// <summary>
            ///     保有するアイテムを捨てます。
            /// </summary>
            /// <param name="id" type="number">
            ///     削除するアイテムのIDを指定
            /// </param>
            /// <param name="count" type="number">
            ///     削除する数を指定（数が０以下ならリストから削除します)
            /// </param>
            /// <return type="boolean">
            var items = this.items;
            for (var i = 0; i < items.length; i++) {
                if (items[i].uid == id) {
                    if (items[i].count <= count) {
                        items.splice(i, 1);
                    } else {
                        items[i].count -= count;
                    }
                    return true;
                }
            }

            return false;
        },
        command: null,
        buy: function (item) {
            /// <summary>
            /// アイテムを購入
            /// </summary>
            /// <param name="item" type="DQ.RPG.Item">
            /// 購入するDQ.RPG.Itemを指定
            /// </param>

            if (world.party.G < item.Buy) {
                return false;
            }

            world.party.G -= item.Buy;
            if (item.category == DQ.RPG.Item.Category.Item) {
                for (var i = 0; i < this.items.length; i++) {
                    if (this.items[i].uid == item.uid) {
                        this.items[i].count++;
                        return this.items[i].serial;
                    }
                }
            }
            this.items.push({ uid: item.uid, count: 1, serial: item.serial, type: item.type, text: item.text });

            return item.serial;
        },
        sell: function (item) {
            /// <summary>
            /// アイテムを売却する
            /// </summary>
            /// <param name="item" type="DQ.RPG.Item">
            /// 売却するDQ.RPG.Itemを指定
            /// </param>

            //売り飛ばせるのか判定が必要
            //            if (0) {
            //                return false;
            //            }

            world.party.G += item.Sell;
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i].uid == item.uid) {
                    if (this.items[i].count <= 1) {
                        this.items.splice(i, 1);
                    } else {
                        this.items[i].count--;
                    }
                    return true;
                }
            }
            return false;
        },
        hasItem: function (uid) {
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i].uid == uid) {
                    return true;
                }
            }

            return false;
        },
        useItem: function (uid, count) {
            /// <summary>
            ///     アイテムを使用します
            /// </summary>
            /// <param name="id" type="number">
            ///     使用するアイテムのIDを指定
            /// </param>
            /// <return type="boolean">
            var items = this.items;
            count = count == undefined ? 1 : count;
            for (var i = 0; i < items.length; i++) {
                if (items[i].uid == uid) {
                    if (items[i].count <= count) {
                        items.splice(i, 1);
                    } else {
                        items[i].count -= count;
                    }
                    return true;
                }
            }

            return false;
        }
    });

    DQ.RPG.Monster = dqextend(DQ.RPG.Creature, function (name, options) {
        this.base(name, options || num);

        options = DQ.options(options, {
            uid: -1,
            name: "",
            text: "",
            image: "mon/back.png",
            description: "",
            HP: 0, MP: 0, STR: 0, VIT: 0, AGI: 0, AI: 0, resistance: [0, 15, 0, 1, 0, 0, 0, 0, 0, 0], action: [0, 5, 0, 5, 0, 5, 0, 5],
            G: 0, EXP: 0, items: [-1, -1, -1, -1, -1], drop: 0
        });

        this.uid = options.uid;
        this.text = name;
        this.title = this.text;
        this.description = options.description;
        this.pname = world.project && world.project.name + "/" || "";
        this.image = this.pname + options.image;
        this.resistance = options.resistance;
        this.action = options.action;
        this.AI = options.AI;
        this.items = options.items;
        this.drop = options.drop;
    }, {
        friend: false,
        displayName: function () {
            return this.text;
        }
    });

});
