﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="dq-ui.js" />

/*
** DQ Retro game UI framework JavaScript library version v0.6.0
**
** Copyright (c) 2009 M., Koji
** Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
**
** Date: 2009-11-01 17:34:21 -0900
** Revision: 0003
**
**/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {
    DQ.UI.Acodion = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// アコーディオンコントロールを作成します。
        /// </summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        /// <param name="object" type="string/jQuery">
        /// アコーディオン コントロールの外殻を指すオブジェクトを指定
        /// </param>

        //options 省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-acodion');
        options = options || {};

        this.base(parent, object);
        this.name = options.name || (typeof object == "string") ? object : object.attr('id');

        this._create(options);
    },
{
    _create: function (options) {
        this.controls = new Array();
        this.tabPages = new Array();

        this.tab = $('>dl', this.obj);
        if (!this.tab.length) {
            this.tab = $('<dl>');
        }
        this.client = this.tab.appendTo(this.obj);

        var me = this;
        $('dt', this.tab).each(function () {
            var dt = $(this),
                dd = dt.next(),
                name = dt.text();

            dt.text('');
            dd = dd[0].nodeName == "DD" ? dd : $('<dd>').appendTo(me.tab);
            dd[0].id = dd[0].id || "acodion_tab_" + DQ.UI.Acodion.count++;
            me._add(new DQ.UI.AcodionPage(me, { 'name': name }, dd[0].id), dt, dd);
        });

        var tabs = options.tabs || {};
        for (var nm in tabs) {
            var dt = $('<dt>').attr('id', nm).appendTo(me.tab),
                dd = $('<dd>').appendTo(me.tab).attr('id', "acodion_tab_" + DQ.UI.Acodion.count++);

            me._add(new DQ.UI.AcodionPage(me, { 'name': tabs[nm].name }, dd[0].id), dt, dd);
        }

        if (this.tabPages.length) {
            var h = this.tabPages[0]._tab.height() * this.tabPages.length;
            for (var i = 0; i < this.tabPages.length; i++) {
                me.tabPages[i].obj.height(me.obj.height() - h);
            }
            this.selectedIndex(0);
        }
    },
    _add: function (tab, dt, dd, div) {
        var me = this;
        if (!dt) {
            dt = $('<dt>').appendClass('col').appendTo(this.tab);
        }
        if (!dd) {
            dd = $('<dd>').appendTo(this.tab);
            div = $('<div>').appendTo(dd);
        }
        me.controls.push(tab);
        me.tabPages.push(tab);
        tab.tab(dt, dd);
    },
    onSelectedIndexChanged: null,
    _selectedIndex: -1,
    selectedIndex: function (idx) {
        if (!arguments.length) {
            return this._selectedIndex;
        }
        if (this.tabPages.length <= idx) {
            throw new RangeError("Indexの範囲を超えています。");
        }
        this._selectedIndex = idx;

        var c = this.tabPages;
        for (var i = 0; i < c.length; i++) {
            if (i == idx) {
                continue;
            }
            c[i].hide();
        }
        c[idx].show();
        this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
    },
    tabPages: null,
    dqid: "DQ.UI.Acodion"
},
{
});
    DQ.UI.Acodion.count = 1;
    DQ.UI.AcodionPage = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// アコーディオンページコントロールを作成します。
        /// </summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        /// <param name="object" type="string/jQuery">
        /// アコーディオンページ コントロールの外殻を指すオブジェクトを指定
        /// </param>

        if (name == "" && !object) {
            throw DQ.InvalidArgument("要素を指定しない場合名前を指定してください。");
        }

        object = object || $('<dd>').attr('id', name);
        options = options || {};

        this.base(parent, object);
        this._create(options);
    }, {
        _create: function (options) {
            this.name = options.name;
        },
        dispose: function () {
            this._tab && this._tab.unbind("click");
        },
        dqid: "DQ.UI.AcodionPage",
        _tab: null,
        tab: function (dt, dd) {
            var me = this;
            this._tab && this._tab.unbind("click");
            this._tab = dt;
            $('<span>').addClass('icon-outer').append($('<span>').addClass('icon').html("&nbsp;")).appendTo(dt);
            $('<span>').addClass('text').text(this.name).appendTo(dt);
            $('<span>').attr('clear', "both").appendTo(dt);

            this.client = dd;
            var _select = function (e) {
                var me = e.data;
                var c = me.parent.tabPages;
                for (var i = 0; i < c.length; i++) {
                    if (c[i] == me) {
                        me.parent.selectedIndex(i);
                        break;
                    }
                }
            }
            this._tab.bind("click", this, _select);
        },
        show: function () {
            this._visible = true;
            this.obj.slideDown();
            this._tab && this._tab.removeClass('col').addClass('ela');
        },
        hide: function () {
            this._visible = false;
            this.obj.slideUp();
            this._tab && this._tab.removeClass('ela').addClass('col');
        }
    });

    DQ._ctrl_['dq-acodion'] = DQ.UI.Acodion;
});
