﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="ui/dq-ui.js" />

/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009-2010 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2010-04-01 17:34:21 -0900
* Revision: 0003
*/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {
    DQ.UI.ListViewItemCollection = dqextend(Array, function (parent, options) {
        /// <summary>
        /// ListViewコントロールの項目のコレクションを表します。
        /// </summary>

        this.base(options);
        this.parent = parent;
        if (parent instanceof DQ.UI.ListView) {
            this.listView = parent;
        } else if (parent instanceof DQ.UI.ListViewGroup) {
            this.listView = parent.parent;
        }

        return this;
    }, {
        obj: null,
        add: function (listViewItem, imageIndex) {
            return this.push(listViewItem, imageIndex);
        },
        addRange: function (value) {
            for (var i = 0; i < value.length; i++) {
                this.push(value[i]);
            }
        },
        push: function (listViewItem, imageIndex) {
            if (typeof listViewItem == "string" && this.parent instanceof DQ.UI.ListView) {
                listViewItem = new DQ.UI.ListViewItem(this.listView, listViewItem);
            } else if (listViewItem instanceof Array && this.parent instanceof DQ.UI.ListView) {
                listViewItem = new DQ.UI.ListViewItem(this.listView, listViewItem);
            }
            this.__super.push.apply(this, [listViewItem]);
            imageIndex != null && listViewItem.imageIndex(imageIndex);

            if (listViewItem.parent instanceof DQ.UI.ListViewGroup) {
                return listViewItem;
            }

            listViewItem._listView = this.listView;
            this.parent.add(listViewItem);

            return listViewItem;
        },
        bind: function (url) {
            var b = this.parent._text;
            this.parent.text("データを読み込みます");
            this._clear();
            var me = this;
            setTimeout(function () {
                DQ.loadCatalog(url, function (data) {
                    me.parent.text(b);
                    eval("var s = " + data);
                    me.json(s);
                });
            }, 15.625);
        },
        remove: function (object) {
            for (var i = 0; i < this.length; i++) {
                if (this[i] == object) {
                    this.splice(i, 1);
                    break;
                }
            }
        },
        json: function (data) {
            for (var nm in data) {
                var item = new DQ.UI.listViewItem(this.listView, nm);
                this.push(item);
                item.json(data[nm]);
            }
        },
        _clear: function () {
            for (var i = 0; i < this.length; i++) {
                this[i].dispose();
            }
            this.length = 0;
        },
        dispose: function () {
            this._clear();
            this.obj.remove();
        }
    });

    DQ.UI.ListView = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// ListViewコントロールを作成します。
        /// </summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        /// <param name="object" type="jQuery">
        /// ListViewコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //options 省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-listview');

        var _options = function (options) {
            options = options || {};
            options.largeImageList = options.largeImageList || new DQ.UI.ImageList();
            options.smallImageList = options.smallImageList || new DQ.UI.ImageList();
            options.view = options.view == null ? DQ.UI.View.Tile : options.view;
            return options;
        }

        options = _options(options);

        this.base(parent, object);
        this.name = options.name || (typeof object == "string") ? object : object.attr('id');
        this.obj.addClass('dq-listview');
        this._groups = [];

        this._create(options);

        this.show();
        return this;
    }, {
        onAfterSelect: null,
        _create: function (options) {
            this._setupCss();

            this.items = new DQ.UI.ListViewItemCollection(this);
            this.largeImageList(options.largeImageList);
            this.smallImageList(options.smallImageList);

            this.tileSize = options.tileSize || new DQ.Size('auto', 'auto');
            this.view(options.view);

            var c = this.obj.children();
            this._defaultGroup = new DQ.UI.ListViewGroup(this, { name: _("Default") });
            for (var i = 0; i < c.length; i++) {
                this.parseHtml(c[i]);
            }
        },
        _setupCss: function () {
            this.styleSheet = $('link[href$="dq-listview.css"]');
            if (!this.styleSheet.length) {
                this.styleSheet = load("dq-listview.css");
            }
            this.styleSheet.attr('id', "dq_css_listview");
            DQ.link(DQ.cssdir + "dq-listview-tile-single.css");
            DQ.link(DQ.cssdir + "dq-listview-list.css");
            DQ.link(DQ.cssdir + "dq-listview-large.css");
            DQ.link(DQ.cssdir + "dq-listview-small.css");
        },
        append: function (element) {
            if (element.dqid) {
                this.client.append(element.obj);
                this.controls.push(element);
            } else {
                this.client.append(element);
            }
            this._addControls(this, this.controls.length - 1);
        },
        add: function (listViewItem) {
            /// <summary>
            /// コレクションへの追加
            /// </summary>

            listViewItem.setupView();
            if (this._showGroup) {
                if (listViewItem.group() == null || !this._groups[listViewItem.group().header()]) {
                    this._defaultGroup.items.push(listViewItem);
                    !this._defaultGroup.visible() && this._defaultGroup.show();
                    return;
                }
                this._groups[listViewItem.group().header()].items.push(listViewItem);
            } else {
                this.append(listViewItem);
            }

            (this.sorting != DQ.UI.SortOrder.None || this._listViewSorter)
        && this.sort();
        },
        dqid: "DQ.UI.ListView",
        findItemWithText: function (text) {
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i].text() == text) {
                    return this.items[i];
                }
            }

            return null;
        },
        groups: function () {
            return this._groups;
        },
        selectedItems: function () {
            var list = [];
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i]._selected) {
                    list.push(this.items[i]);
                }
            }

            return list;
        },
        selectedIndices: function () {
            var list = [];
            for (var i = 0; i < this.items.length; i++) {
                if (this.items[i]._selected) {
                    list.push(i);
                }
            }

            return list;
        },
        _listViewSorter: null,
        listViewSorter: function (value) {
            if (!arguments.length) {
                return this._listViewSorter;
            }
            this._listViewSorter = value;
        },
        _sorting: DQ.UI.SortOrder.None,
        sorting: function (value) {
            if (!arguments.length) {
                return this._sorting;
            }
            if (this._sorting == value) {
                return;
            }
            this._sorting = value;
            this.sort();
        },
        sort: function () {
            var fn = null,
            as = function (a, b) { return a.text() > b.text() ? 1 : -1; },
            de = function (a, b) { return a.text() < b.text() ? 1 : -1; };

            if (this._sorting == DQ.UI.SortOrder.None) {
                fn = this._listViewSorter;
            } else {
                switch (this._sorting) {
                    case DQ.UI.SortOrder.Ascending:
                        fn = as;
                        break;
                    case DQ.UI.SortOrder.Descending:
                        fn = de;
                        break;
                }
            }
            if (fn) {
                this.items.sort(fn);
                this._updateGroup();
            }
        },
        show: function () {
            __dqsuper__(this, 'show', arguments);

            if (this._defaultGroup.items.length > 0) {
                this._defaultGroup.show();
            } else {
                this._defaultGroup.hide();
            }
        },
        _showGroup: true,
        showGroup: function (value) {
            /// <summary>
            /// グループの表示、非表示を取得または設定します。
            /// </summary>
            if (!arguments.length) {
                return this._showGroup;
            }
            this._showGroup = value;
            this._updateGroup();
        },
        _updateGroup: function () {
            var nm, i;
            if (this._showGroup) {
                for (nm in this._groups) {
                    if (this._groups.hasOwnProperty(nm)) {
                        this._groups[nm].show();
                    }
                }
                this._defaultGroup.items.length > 0 && this._defaultGroup.show();
                for (i = 0; i < this.items.length; i++) {
                    var gp = this.items[i].group(),
                    g$ = (gp == null || !this._groups[gp.header()]) ?
                            this._defaultGroup : this._groups[gp.header()];
                    this.items[i].obj.appendTo(g$.client);
                }
            } else {
                this._defaultGroup.hide();
                for (nm in this._groups) {
                    this._groups.hasOwnProperty(nm) && this._groups[nm].hide();
                }
                for (i = 0; i < this.items.length; i++) {
                    this.items[i].obj.appendTo(this.client);
                }
            }
        },
        imageURL: "",
        _imageIndex: 0,
        imageIndex: function (value) {
            ///<summary>
            /// ImageListのimageの位置を指定します。
            ///</summary>
            if (!arguments.length) {
                return this._imageIndex;
            }
            this._imageIndex = value;
            for (var i = 0; i < this.items.length; i++) {
                this.items[i]._setupPosition();
            }
        },
        _largeImageList: null,
        largeImageList: function (value) {
            if (!arguments.length) {
                return this._largeImageList;
            }
            if (typeof value == "string") {
                this._largeImageList = new DQ.UI.ImageList(value);
            } else {
                this._largeImageList = value;
            }
        },
        parseHtml: function (dom) {
            if (dom.nodeName != "DL" && (dom.nodeName != "DIV" || dom.className != "dq-lv-group")) {
                return;
            }
            var me = this,
            largeImageList = this.largeImageList();
            if (dom.nodeName == "DIV") {

            }
            if (dom.nodeName == "DIV" && dom.className == "dq-lv-group") {
                var group = new DQ.UI.ListViewGroup(this);
                this._groups[group.text()] = group;
                me = group;
            }
            var c = $(dom).children();
            var item = null;
            for (var i = 0; i < c.length; i++) {
                var ci = c[i],
                c$ = $(ci);

                if (ci.nodeName == "H2") {
                    me.text($.trim(c$.text()));
                } else if (ci.nodeName == "DL") {
                    me.parseHtml(ci);
                } else if (ci.nodeName == "DT") {
                    item = this.items.add(new DQ.UI.ListViewItem(me, $.trim(c$.text())));
                } else if (ci.nodeName == "DD") {
                    var img = $('img', ci);
                    if (img.length) {
                        largeImageList.urls.push(img[0].src);
                        item && item.imageIndex(largeImageList.urls.length - 1);
                        img.remove();
                    }
                    item && item.subItems.add(c$.html());
                }
            }
            $(dom).remove();
        },
        singleClick: true,
        smallImageList: function (value) {
            if (!arguments.length) {
                return this._imageList;
            }
            if (typeof value == "string") {
                this._imageList = new DQ.UI.ImageList(value);
            } else {
                this._imageList = value;
            }
        },
        items: null,
        tileSize: new DQ.Size(0, 0),
        _view: DQ.UI.View.Tile,
        view: function (value) {
            if (!arguments.length) {
                return this._view;
            }
            this._view = value;
            switch (this._view) {
                case DQ.UI.View.Tile:
                    this.client.removeClass('small-icon large-icon list').addClass('tile-single');
                    break;
                case DQ.UI.View.LargeIcon:
                    this.client.removeClass('small-icon tile-single list').addClass('large-icon');
                    break;
                case DQ.UI.View.SmallIcon:
                    this.client.removeClass('tile-single large-icon list').addClass('small-icon');
                    break;
                case DQ.UI.View.List:
                    this.client.removeClass('tile-single large-icon small-icon').addClass('list');
                    break;
            }
            for (var i = 0; i < this.items.length; i++) {
                this.items[i].setupView();
            }
        }
    });

    DQ.UI.ListViewGroup = dqextend(DQ.Control, function (parent, options) {
        options = options || {};
        this.base(parent, $('<div>').addClass("dq-lv-group"));
        this._header = $('<div>').appendTo(this.obj);
        this.client = $('<div>').appendTo(this.obj);
        this._image = $('<span unselectable="on">&nbsp;</span>').appendTo(this._header).addClass('dq-lvg-expand');
        this._title = $('<h2 />').addClass('dq-lv-title').appendTo(this._header).text(options.name || "");
        this._header.bind('click', this, function (e) {
            var me = e.data;
            me._expanded = !me._expanded;
            me._toggle();
        });

        this.items = new DQ.UI.ListViewItemCollection(this);
    }, {
        add: function (listViewItem) {
            this.client.append(listViewItem.obj);
            listViewItem.group(this);
        },
        dqid: "DQ.UI.ListViewGroup",
        expanded: function (value) {
            if (!arguments.length) {
                return this._expanded;
            }
            if (this._expanded == value) {
                return;
            }
            this._toggle();
        },
        _toggle: function () {
            if (this._expanded) {
                this._image.removeClass('dq-lvg-expand').addClass('dq-lvg-collspan');
                this.client.hide();
            } else {
                this._image.removeClass('dq-lvg-collspan').addClass('dq-lvg-expand');
                this.client.show();
            }
        },
        items: null,
        parseHtml: function (dom) {
            if (dom.nodeName != "DL") {
                return;
            }
            var me = this,
            largeImageList = this.parent.largeImageList();

            var c = $(dom).children();
            var item = null;
            for (var i = 0; i < c.length; i++) {
                if (c[i].nodeName == "H2") {
                    me.text($(c[i]).text());
                    continue;
                }
                if (c[i].nodeName == "DT") {
                    item = me.items.add(new DQ.UI.ListViewItem(me, $.trim($(c[i]).text())));
                    me.parent.items.add(item);
                    continue;
                }
                if (c[i].nodeName == "DD") {
                    var img = $('img', c[i]);
                    if (img.length) {
                        largeImageList.urls.push(img[0].src);
                        item && item.imageIndex(largeImageList.urls.length - 1);
                        img.remove();
                    }
                    item && item.subItems.add($(c[i]).html());
                }
            }
            $(dom).remove();
        },
        header: function (value) {
            if (!arguments.length) {
                return this._title.text();
            }
            this._title.text(value);
        },
        text: function (value) {
            if (!arguments.length) {
                return this._title.text();
            }
            this._title.text(value);
        }
    });
    DQ.UI.ListViewItem = dqextend(DQ.Control, function (parent, text, options) {
        var object = $('<div class="dq-listviewitem" />');
        this.base(parent, object);
        this.client = $('<div class="dq-lvi-inner" />').appendTo(this.obj);
        if (text instanceof Array) {
            this._text = text.shift();
        } else {
            this._text = text;
        }
        var listView = (parent instanceof DQ.UI.ListViewGroup) ? parent.parent : parent;
        this._create(listView, options);
        if (text instanceof Array) {
            this.subItems.addRange(text);
        }
    }, {
        onAfterClick: null,
        _afterClick: function (node, e) {

        },
        add: function (text) {
            ///<summary>
            /// サブアイテムの追加
            ///</summary>
            this._ddObj.html(this.subItems.join(''));
        },
        _create: function (listView, options) {
            this._listView = listView;
            this.subItems = new DQ.UI.ListViewItemCollection(this);
            this.subItems.listView = listView;

            this._imgObj = $('<img class="image" />').appendTo(this.client);
            this._txtObj = $('<dl class="text" />').appendTo(this.client).append('<dt>' + this._text + '</dt>');
            this._ddObj = $('<dd />').appendTo(this._txtObj).hide();
            $('<div style="clear: both;height: 0; visibility: hidden;" />').appendTo(this.client);

            var me = this;
            this.client.hover(function () {
                if ($(this).hasClass('dq-lvi-selected')) {
                    return;
                }
                $(this).addClass('dq-lvi-hover');
                if (me._listView.view() == DQ.UI.View.Tile && me._listView.singleClick) {
                    me._ddObj.show();
                }
            }, function () {
                $(this).removeClass('dq-lvi-hover');
                if (me._listView.view() == DQ.UI.View.Tile && me._listView.singleClick) {
                    me._ddObj.hide();
                }
            })
        .bind('click', this, function (e) {
            var me = e.data,
                lv = me._listView;
            $('.dq-lvi-inner', lv.client).removeClass('dq-lvi-selected');
            $('dd', lv.client).hide();

            if (!lv.singleClick) {
                $(this)
                .addClass('dq-lvi-selected')
                .removeClass('dq-lvi-hover');
                if (!$(this).parent().hasClass('dq-lvi-list')) {
                    $('dd', this).show();
                }
            }
            lv.onClick && lv.onClick(lv, me);
        });

            this._listView && this.setupView();
            return this;
        },
        checked: function () {
            return false;
        },
        _group: null,
        group: function (value) {
            if (!arguments.length) {
                return this._group;
            }
            if (this._group == value) {
                return;
            }
            if (this._group != null) {
                this._group.items.remove(this);
            }
            this._group = value;
            this._group.items.add(this);

        },
        setupView: function () {
            switch (this._listView.view()) {
                case DQ.UI.View.Tile:
                    this.client
                .width(this._listView.tileSize.width)
                .height(this._listView.tileSize.height);
                    if (this.imageIndex() >= 0) {
                        this._imgObj.attr('src', this.imageList().urls[this.imageIndex()]);
                    }
                    break;
                case DQ.UI.View.LargeIcon:
                    this.client.css({ 'width': null, 'height': null });
                    if (this.imageIndex() >= 0) {
                        this._imgObj.attr('src', this.imageList().urls[this.imageIndex()]);
                    }
                    break;
                case DQ.UI.View.SmallIcon:
                    this.client.css({ 'width': null, 'height': null });
                    if (this.imageIndex() >= 0) {
                        this._imgObj.attr('src', this.imageList().urls[this.imageIndex()]);
                    }
                    break;
                case DQ.UI.View.List:
                    this.client.css({ 'width': null, 'height': null });
                    if (this.imageIndex() >= 0) {
                        this._imgObj.attr('src', this.imageList().urls[this.imageIndex()]);
                    }
                    break;
            }
        },
        dispose: function () {
            this._expObj.unbind("click");
            this._txtObj.unbind("click");
            this.subItems.dispose();
            __dqsuper__(this, "dispose");
        },
        dqid: "DQ.UI.ListViewItem",
        index: function () {
            var p = this._listView;
            for (var i = 0; i < p.items.length; i++) {
                if (p.items[i] == this) {
                    return i;
                }
            }

            return -1;
        },
        _setupPosition: function (disable) {
            var index = this._imageIndex ? this._imageIndex : this._listView.imageIndex();
            if (index == -1) {
                this._imgObj.hide();
            } else {
                this._imgObj.attr('src', this.imageList().urls[this.imageIndex()]);
            }
        },
        _imageIndex: -1,
        imageIndex: function (pos) {
            if (!arguments.length) {
                return this._imageIndex;
            }
            this._imageIndex = pos;
            this._listView && this._setupPosition(true);
        },
        imageList: function () {
            if (this._view == DQ.UI.View.SmallIcon) {
                return this._listView.smallImageList();
            } else {
                return this._listView.largeImageList();
            }
        },
        json: function (data) {
            if (typeof data == "string") {
                this.url = data;
            } else {
                this.subItems.json(data);
            }
        },
        _selected: false,
        selected: function (value) {
            if (!arguments.length) {
                return this._selected;
            }

            if (!value) {
                this._txtObj.removeClass('dq-lvi-selected');
            } else {
                this._txtObj.addClass('dq-lvi-selected');
            }
            this._selected = value;
        },
        subItems: null,
        text: function (value) {
            if (!arguments.length) {
                return this._text;
            }
            this._text = data;
            this._txtObj.text(data);
        },
        _listView: null,
        listView: function () {
            if (this._listView != null) {
                return this._listView;
            }
            var p = this.parent;
            while (p != null) {
                if (p._listView != null) {
                    this._listView = p._listView;
                    break;
                }
                p = p.parent;
            }
            return this._listView;
        }
    });

    DQ._ctrl_['dq-listview'] = DQ.UI.ListView;
});
