﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="../dq-core.js" />
/// <reference path="dq-ui.js" />
/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: (http://www.opensource.org/licenses/mit-license.php)
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0003
*/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {

    DQ.Size.prototype.toString = function () { return [this.width, this.height].join(','); }
    DQ.Control.prototype.toString = function () { return "[" + this.dqid + "]"; }
    DQ.UI.UITypeEditorEditStyle = { none: 0, modal: 1, DropDown: 2, inline: 3 }
    DQ.UI.TypeEditor = function () {
        ///<summary>
        /// 型に応じた編集機能を提供します。
        ///</summary>
    }

    DQ.UI.TypeEditor.prototype = {
        getInput: function (type, nm, ro, value) {
            ///<summary>
            /// インラインの編集要素を取得します。
            ///</summary>
            ///<param name="type" type="string">
            /// valueの型を指定します。
            ///</param>
            ///<param name="nm" type="string">
            /// 編集対象の名前を指定
            ///</param>
            ///<param name="ro" type="boolean">
            /// 読み取り専用ならtrueを指定
            ///</param>
            ///<param name="value" type="object">
            /// 編集対象の値を指定
            ///</param>

            return $('<input type="TEXT" p_type="' + type + '" id="' + nm + '" value="' + value + '" ' + ro + ' />');
        },
        hasCustomEditor: function () {
            ///<summary>
            /// カスタムエディターの有無を取得します。
            ///</summary>
            return false;
        },
        getCustomEditor: function (type, nm, value) {
            ///<summary>
            ///input以外のカスタムエディターを返却します。
            ///</summary>
            return undefined;
        }
    }

    DQ.UI.TextAreaEditor = dqextend(DQ.UI.TypeEditor, function (options) {
        ///<summary>
        ///複数行のテキスト編集エディター
        ///</summary>

        options = options || {};
        this.rows = options.rows ? options.rows : 3;
    }, {
        getInput: function (type, nm, ro, value) {
            return $('<textarea p_type="' + type + '" id="' + nm + '" ' + ro + '>' + value + '</textarea>').css('height', this.rows * 16);
        }
    });

    DQ.UI.BooleanEditor = dqextend(DQ.UI.TypeEditor, function () {
        ///<summary>
        ///Booleanエディター
        ///</summary>
    }, {
        hasCustomEditor: function () {
            return true;
        },
        getCustomEditor: function (type, nm, value) {
            var s = $('<select p_type="' + type + '" id="s_' + nm + '" name="' + nm + '"></select>');
            s.append($('<option>true</option><option>false</option>'));
            return s;
        }
    });

    DQ.UI.DropDownEditor = dqextend(DQ.UI.TypeEditor, function (values) {
        ///<summary>
        ///Booleanエディター
        ///</summary>
        this._values = values;
    }, {
        hasCustomEditor: function () {
            return true;
        },
        getCustomEditor: function (type, nm, value) {
            var s = $('<select p_type="' + type + '" id="s_' + nm + '" name="' + nm + '"></select>');
            for (var i = 0; i < this._values.length; i++) {
                s.append($('<option>').text(this._values[i]));
            }
            return s;
        }
    });

    DQ.UI.PropertyGrid = dqextend(DQ.Control, function (parent, options, object) {
        /// <summary>
        /// 渡されたオブジェクトの内容を一覧します。
        /// </summary>
        ///	<param name="parent" type="DQ.Control">
        ///		親コントロールを指定
        ///	</param>
        ///	<param name="options" type="object">
        ///		PropertyGridコントロールへの設定値を指定
        ///	</param>
        /// <param name="object" type="jQuery">
        /// PropertyGridコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //options 省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;

        object = object || $('<div>').addClass('dq-property-grid').width('8em').height('9em');
        options = options || {};

        this.base(parent, object);
        this._create(parent, options);

        this.setContent(this, true);
    }, {
        _create: function (parent, options) {
            ///<summary>
            /// コントロールの外観を作成します。
            ///</summary>
            ///	<param name="parent" type="DQ.Control">
            ///		親コントロールを指定
            ///	</param>
            ///	<param name="options" type="object">
            ///		PropertyGridコントロールへの設定値を指定
            ///	</param>

            this.parent = parent;
            this._onlyLiteral = (options.onlyLiteral == null) ? true : options.onlyLiteral;
            this._readOnly = (options.readOnly == null) ? false : options.readOnly;
            options.selectedObject && this.setContent(options.selectedObject, this._onlyLiteral);
        },
        _change: function (pl, sender) {
        },
        dqid: "DQ.UI.PropertyGrid",
        dispose: function () {
            $.removeData(this.obj);
            return this;
        },
        meta: null,
        setContent: function (target, onlyLiteral, gtb, recursive) {
            this._selectObject = target;
            this.table && this._remove();
            this.table = $('<table>').attr('border', 0).appendTo(this.client);
            gtb = this.table;
            if (!gtb) {
            }

            var me = this,
            onlyLiteral = onlyLiteral || me._onlyLiteral,
            group = [],
            srt = [],
            meta = this.meta,
            names = srt.sort(function (a, b, x, y) {
                x = a.charAt(0) == "_" ? a.substring(1) : a;
                y = b.charAt(0) == "_" ? b.substring(1) : b;
                return (x == y) ? 0 : (x < y ? -1 : 1);
            }),
            _setInput = function (nm, value, type, ro, ronly, editor, td) {
                var d = $('<div>').appendTo(td);
                if (me._readOnly || ronly) {
                    ro = 'disabled="disabled"';
                }
                d.append(editor.getInput(type, nm, ro, value));
                if (editor.hasCustomEditor()) {
                    d.append(editor.getCustomEditor(type, nm, value).hide());
                }
            },
            _appendGroup = function (target, table, nm, dn, ronly, editor) {
                var value = false,
                    lite = false,
                    str,
                    tr,
                    td;
                if (nm.charAt(0) == "_" && nm.charAt(1) == "_") {
                    //プライベートな属性値なら終了
                    return;
                }
                if (target[nm] == null) {
                    str = "{null}";
                } else if (typeof target[nm] == "object") {
                    var type = (typeof target[nm]);
                    if (nm.charAt(0) == "_") {
                        if (target[nm.substring(1)] != null) {
                            nm = nm.substring(1);
                            value = target[nm].apply(target);
                            lite = true;
                        }
                    } else {
                        value = target[nm];
                        lite = true;
                    }

                    value = value != null && (value.toString ? value.toString() : "{object}") || "{object}";
                } else if (target[nm] instanceof Array) {
                    value = target[nm].toString ? target[nm].toString() : "[array]";
                } else if (typeof target[nm] == "function") {
                    value = "{...}";
                } else {
                    var type = (typeof target[nm]);
                    var _nm = "_" + nm;
                    var ro = "";
                    if (nm.charAt(0) == "_") {
                        if (target[nm.substring(1)] != null) {
                            nm = nm.substring(1);
                            value = target[nm].apply(target);
                            ro = target[_nm] && !target[_nm].set ? 'disabled="disabled"' : "";
                            lite = true;
                        }
                    } else {
                        value = target[nm];
                        lite = true;
                    }
                }
                if (lite || !onlyLiteral) {
                    tr = $('<tr>').appendTo(table);
                    $('<th>').html(dn).appendTo(tr);
                    td = $('<td>').appendTo(tr);

                    //入力タグを設定
                    _setInput(nm, value, type, ro, ronly, editor, td);
                }

                return tr;
            },
            change = function _change(sender) {
                me._selectObject[$(sender).content.id] = $(sender).content.value;
            }

            //ソート用の名前配列を作成
            for (var nm in target) {
                srt.push(nm);
            }
            //メタ情報からグループを作成
            var gtr = $('<tr>')
                .append($('<td>').addClass("colapse").append($('<span>').addClass("dq-treeimage").html("&nbsp;")))
                .append($('<td>').addClass("group").text("その他"))
                .appendTo(gtb),
                gtr = $('<tr>').append($('<td>').addClass("colapse")).appendTo(gtb),
                gtd = $('<td>').addClass("cont").appendTo(gtr);

            group.push($('<table>').appendTo(gtd));
            if (meta) {
                for (var nm in meta) {
                    if (meta[nm].category && !group[meta[nm].category]) {
                        gtr = $('<tr>')
                        .append($('<td>').addClass("colapse").append($('<span class="dq-treeimage">&nbsp;</span>')))
                        .append($('<td>').addClass("group").text(meta[nm].category))
                        .appendTo(me.table);
                        gtr = $('<tr>').appendTo(me.table);
                        $('<td>').addClass("colapse").appendTo(gtr);
                        gtd = $('<td>').addClass("cont").appendTo(gtr);
                        group[meta[nm].category] = $('<table>').appendTo(gtd);
                    }
                }
            }
            $('.dq-treeimage', this.obj).bind("click", function (e) {
                var p = $(this).parent().parent().next();
                if ($(this).hasClass('dq-col')) {
                    p.show();
                    $(this).removeClass('dq-col');
                } else {
                    p.hide();
                    $(this).addClass('dq-col');
                }
            });
            // table 作成
            var hasMisc = false;
            this.nodes = [];
            for (var i = 0; i < names.length; i++) {
                var nm = names[i],
                str = nm.charAt(0) == "_" ? nm.substring(1) : nm,
                dn = str,
                ed,
                ro = false,
                curGroup,
                tr;

                if (nm.charAt(0) == "_" && nm.charAt(1) == "_") {
                    continue;
                }

                //メタ情報から表示属性を取得
                if (meta && meta[str] && group[meta[str].category]) {
                    dn = meta[str].displayName || str;
                    ro = meta[str].readOnly != null ? meta[str].readOnly : false;
                    curGroup = group[meta[str].category];
                } else {
                    curGroup = group[0];
                }

                //TypeEditor決定
                ed = new DQ.UI.TypeEditor();
                if (meta && meta[str] && meta[str].editor) {
                    ed = meta[str].editor;
                } else {
                    switch (typeof target[nm]) {
                        case "object":
                            ed = new DQ.UI.TypeEditor();
                            dn = '<span class="dq-innerimage" style="width: 9px; height: 9px;">&nbsp;</span>' + dn;
                            break;
                        case "function":
                            ed = new DQ.UI.TypeEditor();
                            break;
                        case "number":
                            ed = new DQ.UI.TypeEditor();
                            break;
                        case "boolean":
                            ed = new DQ.UI.BooleanEditor();
                            break;
                        case "string":
                            ed = new DQ.UI.TypeEditor();
                            break;
                    }
                }
                tr = _appendGroup(target, curGroup, nm, dn, ro, ed);
                if (tr) {
                    tr.attr('id', this.nodes.length);
                    this.nodes.push({ node: tr, parent: null });
                }

                if (nm.charAt(0) == "_" && !tr) {
                    continue;
                }

                if (!(meta && meta[str] && group[meta[str].category])) {
                    target.hasOwnProperty(nm) &&
                     (hasMisc = true);
                }

                //objectなら入れ子表示
                if (!recursive && typeof target[nm] == "object") {
                    for (var snm in target[nm]) {
                        if (!isNaN(snm)) {
                            dn = '<div style="width:16px;">&nbsp;</div>[' + snm + ']';
                        } else {
                            dn = '<div style="width:16px;">&nbsp;</div>' + snm;
                        }
                        var subTR = _appendGroup(target[nm], curGroup, snm, dn, ro, new DQ.UI.TypeEditor());
                        if (subTR) {
                            subTR.attr('id', this.nodes.length);
                            subTR.attr('name', nm);
                            this.nodes.push({ node: subTR, parent: tr, obj: target[nm] });
                        }
                    }
                    if (tr) {
                        $('.dq-innerimage', tr).bind("click", nm, function (e) {
                            var p = $(this).parent().parent().siblings('[name=' + e.data + ']');
                            if ($(this).hasClass('dq-col')) {
                                p.show();
                                $(this).removeClass('dq-col');
                            } else {
                                p.hide();
                                $(this).addClass('dq-col');
                            }
                        }).trigger("click");
                    }
                }
            }
            //その他テーブルを未使用なら非表示にする
            !hasMisc && group[0].parent().parent().hide().prev().hide();

            //
            // イベント付加
            //
            var tables = $('table', gtb);
            tables.each(function () {
                var table = this;
                var txt = $('input[type=text]', table),
                    tr = $('tr', table),
                    td = $('td', table),
                    sel = $('select', table),
                    area = $('textarea', table),
                    pre = null;

                $('th', tr).click(function (e) {
                    $('tr', me.table).removeClass('select');
                    $(this).parent().addClass('select');
                    $('input', $(this).parent()).focus();
                });
                txt.change(function () {
                    me._text_change.apply(me, [this]);
                });
                txt.keypress(function (e) { e.which == 13 && me._text_change.apply(me, [this]); });
                txt.focus(function (e) {
                    if (pre) {
                        $(pre).show();
                        $('#s_' + pre.id).hide();
                        pre = null;
                    }
                    if ($(this).attr('p_type') == "boolean") {
                        var me = $(this);
                        me.hide();
                        var s = $('#s_' + this.id).show().focus();
                        eval("var v =" + this.value);
                        s[0].selectedIndex = v ? 0 : 1;
                        pre = this;
                    } else {
                        var s = $('#s_' + this.id);
                        if (s.length) {
                            var me = $(this);
                            me.hide();
                            s.show().focus();
                            for (var i = 0; i < s[0].options.length; i++) {
                                if (this.value == s[0].options[i].text) {
                                    s[0].selectedIndex = i;
                                    break;
                                }
                            }
                            pre = this;
                        }
                    }
                    this.select(0, this.value.length);
                    return false;
                });
                sel.change(function () {
                    var m = $('#' + this.name);
                    m[0].value = this.options[this.selectedIndex].text;
                    me._text_change(m[0]);
                })
                .focusout(function () {
                    $(this).hide();
                    $('#' + this.name).show();
                    pre = null;
                });
                area.change(function () { me._text_change.apply(me, [this]); });
            });

            return this;
        },
        _remove: function () {
            var tables = $('table', this.table);
            tables.each(function () {
                var table = this;
                var txt = $('input[type=text]', table),
                    tr = $('tr', table),
                    td = $('td', table),
                    sel = $('select', table),
                    area = $('textarea', table);

                tr.unbind("click");
                txt.unbind("change").unbind("keypress").unbind("focusin").unbind("focusout");
                sel.unbind("change");
            });
            this.table.remove();
        },
        refresh: function () {
            this.setContent(this._selectObject, this._onlyLiteral);
        },
        _text_change: function (elem) {
            var obj = this._selectObject,
            tr = $(elem).parent().parent().parent(),
            node = this.nodes[tr.attr('id')];
            if (node.parent) {
                obj = node.obj;
                tr = node.parent;
            }
            var cur = obj[elem.id];
            if (typeof cur == "function") {
                var v = false;
                try {
                    switch ($(elem).attr('p_type')) {
                        case "number":
                            v = parseFloat(elem.value);
                            break;
                        case "boolean":
                            eval("v =" + elem.value);
                            break;
                        case "string":
                            v = this.value;
                            break;
                        case "object":
                            v = elem.value.match(/,/) && elem.value.split(',');
                    }
                } catch (ex) {
                    DQ.UI.MessageBox.show("プロパティ値が不正です");
                    return;
                }
                if (v instanceof Array) {
                    cur.apply(obj, v);
                } else {
                    cur.apply(obj, [v]);
                }
            } else {
                if (obj[elem.id] instanceof Array) {
                    obj[elem.id] = elem.value.split(',');
                    var ar = obj[elem.id];
                    for (var i = 0; i < ar.length; i++) {
                        if (!isNaN(parseFloat(ar[i]))) {
                            ar[i] = parseFloat(ar[i]);
                        }
                    }
                    this.refresh();
                } else if (obj[elem.id] instanceof Object) {
                    return;
                } else if (typeof obj[elem.id] == "number") {
                    obj[elem.id] = parseFloat(elem.value);
                } else if (typeof obj[elem.id] == "boolean") {
                    obj[elem.id] = elem.value ? true : false;
                } else {
                    obj[elem.id] = elem.value;
                }
                if (node.parent) {
                    $('input', tr).attr('value', obj.toString());
                }
            }
        },
        table: null,
        apply: function () {
            var me = this,
            txt = $('input[type=text]', this.obj),
            sel = $('select', this.obj),
            area = $('textarea', this.obj);

            txt.each(function () {
                me._text_change(this);
            });
            area.each(function () {
                me._text_change(this);
            });

        }
    }, {
        readOnly: { get: function () { return this._readOnly; }, set: function (value) { this._readOnly = value; }, def: false },
        onlyLiteral: { get: function () { return this._onlyLiteral; }, set: function (value) { this._onlyLiteral = value; }, def: false },
        selectObject: { get: function () { return this._selectObject; }, set: function (value) { this.setContent(value); }, def: null }
    });

    DQ._ctrl_['dq-property-grid'] = DQ.UI.PropertyGrid;

});