﻿/// <reference path="../jquery.js" />
/// <reference path="../dq.js" />
/// <reference path="dq-ui.js" />
/*
* DQ Retro game UI framework JavaScript library version v0.6.0
*
* Copyright (c) 2009 M., Koji
* Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
*
* Date: 2009-11-01 17:34:21 -0900
* Revision: 0004
*/

DQ.lazyLoad(DQ.subdir + "ui/dq-ui.js", "DQ.UI", function () {

    DQ.UI.Alignment = { Top: 0, Bottom: 1 };

    DQ.UI.TabControl = dqextend(DQ.Control, function (parent, name, options, object) {
        /// <summary>
        /// Tabコントロールを作成します。
        /// </summary>
        ///<param name="options" type="object">
        /// オプションを指定
        ///</param>
        /// <param name="object" type="string/jQuery">
        /// Tabコントロールの外殻を指すオブジェクトを指定
        /// </param>

        //options 省略判定
        object = !object && options && (typeof options == "string" || options.jquery) && options || object;
        object = object || $('<div>').addClass('dq-tabctrl');

        options = DQ.options(options, {
            name: ((typeof object == "string") ? object : object.attr('id')),
            alignment: DQ.UI.Alignment.Top
        });

        this.base(parent, object);
        this.name = options.name;

        this._create(options);
    }, {
        _alignment: DQ.UI.Alignment.Top,
        alignment: function (value) {
            if (!arguments.length) {
                return this._alignment;
            }
            if (this._alignment == value) {
                return;
            }
            this._alignment = value;
            if (value == DQ.UI.Alignment.Top) {
                this.frame.removeClass('dq-tab-bottom').addClass('dq-tab-top');
                this.client.appendTo(this.obj);
            } else {
                this.frame.removeClass('dq-tab-top').addClass('dq-tab-bottom');
                this.frame.appendTo(this.obj);
            }
        },
        _create: function (options) {
            this.controls = new Array();
            this.tabPages = new Array();

            this.frame = $('<div>').addClass('dq-tab-nav dq-tab-top').appendTo(this.obj);
            this.tab = $('>ul', this.obj);
            if (!this.tab.length) {
                this.tab = $('<ul>');
            }
            this.tab.appendTo(this.frame);
            this.client = $('<div>').addClass('dq-tab-panel').appendTo(this.obj);

            var me = this;
            var tabWidth = 0;
            $('li', this.tab).each(function () {
                var a = $('a', this),
                    id = a.attr('href'),
                    text = a.text();
                a.remove();
                $('<div>').addClass('dq-tab-nav-a').text(text).appendTo(this);
                var p = new DQ.UI.TabPage(me, { name: id }, $(id));
                me.add(p, $(this));
                tabWidth += p.tab().outerWidth({ margin: true });
            });
            this.tab.width(Math.max(tabWidth, this.frame.width()));

            var style = this.client[0].currentStyle || document.defaultView.getComputedStyle(this.client[0], ''),
            ph = parseInt(style.paddingTop) + parseInt(style.paddingBottom),
            bh = parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);
            this.client.height(this.obj.height() - this.frame.height() - bh - ph);

            this.alignment(options.alignment);
            this.tabPages.length && this.selectedIndex(0);

            this._rightButton = $('<div>').addClass('dq-tab-right').appendTo(this.frame).hide();
            this._leftButton = $('<div>').addClass('dq-tab-left').appendTo(this.frame).hide();
            this._rightButton.bind('click', this, function (e) {
                var me = e.data;
                var st = 0;
                me._left++;
                if (me._left >= me.tabPages.length - 1) {
                    me._left = me.tabPages.length - 2;
                }
                for (var i = 0; i < me._left; i++) {
                    st += me.tabPages[i].tab().outerWidth({ margin: true });
                }
                me.tab.css('left', -st);
            });
            this._leftButton.bind('click', this, function (e) {
                var me = e.data;
                var st = 0;
                me._left--;
                if (me._left < 0) {
                    me._left = 0;
                }
                for (var i = 0; i < me._left; i++) {
                    st += me.tabPages[i].tab().outerWidth({ margin: true });
                }
                me.tab.css('left', -st);
            });
            if (this.tab.width() > this.frame.width()) {
                this._rightButton.show();
                this._leftButton.show();
            }
        },
        _left: 0,
        add: function (tab, li) {
            var me = this;
            if (!li) {
                li = $('<li>').appendTo(this.tab);
                var a = $('<a>').appendTo(li).attr('href', tab.name);
            }
            me.controls.push(tab);
            me.tabPages.push(tab);
            tab.tab(li);
        },
        resize: function () {
            this.dock(this._dock);
            var style = this.client[0].currentStyle || document.defaultView.getComputedStyle(this.client[0], ''),
            ph = parseInt(style.paddingTop) + parseInt(style.paddingBottom),
            bh = parseFloat(style.borderTopWidth) + parseFloat(style.borderBottomWidth);

            this.client.height(this.obj.height() - this.frame.height() - bh - ph);
        },
        update: function () {
            this.resize();
        },
        onSelectedIndexChanged: null,
        _selectedIndex: -1,
        selectedIndex: function (idx) {
            if (!arguments.length) {
                return this._selectedIndex;
            }
            if (this.tabPages.length <= idx) {
                throw new Error("Indexの範囲を超えています。");
            }
            this._selectedIndex = idx;

            var c = this.tabPages;
            for (var i = 0; i < c.length; i++) {
                if (i == idx) {
                    continue;
                }
                c[i]._tab.removeClass('dq-tab-selected');
                c[i].hide();
            }
            $('li', this.client).removeClass('dq-tab-selected');
            c[idx]._tab.addClass('dq-tab-selected');
            c[idx].show();
            this.onSelectedIndexChanged && this.onSelectedIndexChanged(this);
        },
        tabPages: null,
        dqid: "DQ.UI.TabControl"
    });

    DQ.UI.TabPage = dqextend(DQ.Control, function (parent, options, object) {
        options = DQ.options(options, { name: "" });
        if (options.name == "" && !object) {
            throw new Error("要素を指定しない場合名前を指定してください。");
        }

        object = object || $('<div>').addClass('dq-tabpage').attr('id', name);
        this.base(parent, object);
        this.name = options.name;
        this.hide();
    }, {
        dispose: function () {
            this._tab && this._tab.unbind("click");
        },
        dqid: "DQ.UI.TabPage",
        _tab: null,
        tab: function (li) {
            if (arguments.length == 0) {
                return this._tab;
            }
            var me = this;

            this._tab && this._tab.unbind("click");
            this._tab = li;

            var _select = function (e) {
                var me = e.data,
                c = me.parent.tabPages;
                for (var i = 0; i < c.length; i++) {
                    if (c[i] == me) {
                        me.parent.selectedIndex(i);
                        break;
                    }
                }
            }
            this._tab.bind("click", this, _select);
        }
    });

    DQ._ctrl_['dq-tabctrl'] = DQ.UI.TabControl;
});